#!/bin/bash
#
#  pd-admin V4/SE installation 
#  2004-10-28
#  2004-12-21 stvo
#  2007-10-26 stvo
#  $Id: install-all.sh,v 1.63 2024/04/04 14:47:16 testx Exp testx $
#

set -e
#set -x

if type iconv >/dev/null; then
   lang_saved=$LANG
   function echo ()
   {
      local LANG=$lang_saved
      export LANG
      /bin/echo "$@" | iconv -c -f ISO-8859-15
   }
fi

unset LANG || true
unset LC_ALL || true
unset LC_CTYPE || true
unset LC_COLLATE || true

SEINSTDIR=/usr/local/pd-admin2
PATH=/usr/local/bin:$PATH

echo="echo -n"

HERE=`pwd`
runlevel=`runlevel | cut -d' ' -f2`

function usage ()
{
   echo " "
   echo "install-all.sh [-s Reihe] [-v Version] [--email <address> [--unattended]]"
   echo " "
   echo "  -s Reihe       : Installation der Reihe erzwingen"
   echo " "
}

function unattended_install()
{
#   echo "unattended_install called"
#   sleep 3
   if [ "$unattended" = "" ]; then
      false
   else
      true
   fi
}

mysql_credential_column_name=(
   ''                      # SE-0
   password                # SE-1
   password                # SE-2
   password                # SE-3
   password                # SE-4
   password                # SE-5
   authentication_string   # SE-6
   password                # SE-7
   ''                      # SE-8
   ''                      # SE-9
   ''                      # SE-10
)

user_table_is_updatable=(
   [9]=false
   [10]=false
)

function process_command_line()
{
#   echo "process command line"
   NEWSERIES=
   NEWVERSION=${NEWVERSION:=}
   params=$(getopt -n "$0" -o "s:hv:" -l email:,unattended -- "$@")
   eval set -- "$params"
   while :; do
      case "$1" in
         --)
            shift
            break;;
         -h) usage; exit 0;;
         -v)
            NEWVERSION=$2
            shift;;
         -s)
            NEWSERIES=$2
            shift;;
         --email)
            export email=$2
            shift;;
         --unattended)
            export unattended=1;;
         *) usage; exit 1;;
      esac
      shift
   done
   if [ "$*" != "" ]; then
      echo "argv: <""$@"">"
      usage
      exit 1
   fi
   if unattended_install; then
      if [ "$email" = "" ]; then
         usage
         exit 1
      fi
   fi
}

download_std_serv_env ()
{
	if [ -d ${SEINSTDIR} ]; then
	   echo "WARNUNG: Das Verzeichnis ${SEINSTDIR} existiert"
	   echo "bereits. Sichern Sie den Inhalt des Verzeichnisses und "
	   echo "entfernen Sie es danach. Anschlieend knnen Sie die "
	   echo "Installation durchfhren."
	   exit -1
        fi

   SERIES=
   if [ "$NEWSERIES" == "" ]; then
      echo "Aktuelle Reihe der Server-Umgebung (SE) wird ermittelt"
      rm -f SE-CURRENT-SERIES
      wget -nc http://download.pd-admin.de/SE-CURRENT-SERIES || true
      if [ -f SE-CURRENT-SERIES ]; then
         SERIES=`cat SE-CURRENT-SERIES`
      else
         echo "Konnte SE-CURRENT-SERIES nicht herunterladen"
         exit 771
      fi
   else
      SERIES=$NEWSERIES
   fi

   SEVERSION=
   if [ "$NEWVERSION" == "" ]; then
      echo "Aktuelle Version der Server-Umgebung (SE) wird ermittelt"
      rm -f SE-LATEST
      wget -nc http://download.pd-admin.de/SE-LATEST || true
      if [ -f SE-LATEST ]; then
         SEVERSION=`cat SE-LATEST`
      else
         echo "Konnte SE-LATEST nicht herunterladen"
         exit 772
      fi
   else
      SEVERSION=$NEWVERSION
   fi

   SETGZ="se-$SERIES-$SEVERSION.tar.gz"

	echo "Lade die Server-Umgebung (SE) herunter (ca. 600 MB)"
   wget -nc http://download.pd-admin.de/SE-SERIES/$SETGZ || true
   if [ -f $SETGZ ]; then
      pushd /usr/local 1>/dev/null 2>&1
      echo "Entpacke Server-Umgebung. Bitte haben Sie etwas Geduld."
      tar xfz $HERE/$SETGZ || {
      echo "Kann $SETGZ nicht entpacken"
      exit 2
      }
#
#	Berechtigungen/Eigentmerschaften anpassen
#
      ${SEINSTDIR}/post-install.sh


#
#  apache now controlled by daemontools
#

      popd 1>/dev/null 2>&1
   else
      echo "Server-Umgebung $SETGZ nicht heruntergeladen."
      exit 2
   fi
}

myprompt()
{
 answer="##"
 until echo $1 | grep $answer >/dev/null
 do
    $echo "${2} [${3}]?  ${nnl}"
    read answer
    if [ "$3" != "" ] && [ "$answer" = "" ]
    then
       answer=$3
    fi
 done
}

pp()
{
 prompt=$1
 PASSWORD=""
 until [ "$PASSWORD" != "" ]
 do
    $echo "$prompt:  ${nnl}"
    stty -echo
    read password1
    stty echo
    echo
    $echo "Re-enter $prompt:  ${nnl}"
    stty -echo
    read password2
    stty echo
    echo
    if [ "$password1" = "$password2" ]
    then
        PASSWORD=$password1
    fi
 done
  
 return 1
}

#####
#####   M A I N
#####

mypath=`pwd` 

VERSION=`cat VERSION`

clear
echo "#######################################################"
echo "#                                                     #"
echo "#  Willkommen bei der Installation von pd-Admin $VERSION  #"
echo "#                                                     #"
echo "#######################################################"
echo
echo "Build: `cat BUILD`"
echo 'Installer: $Revision: 1.63 $'

if [ $UID -ne 0 ]; then
   echo "Sie mssen als Superuser (root) angemeldet sein, um pd-admin"
   echo "zu installieren. Bitte melden Sie sich als Superuser an oder"
   echo "wechseln Sie mit su (1) ihre aktuelle user id."
   exit 1
fi

$HERE/tools/install_packages.sh

   BASE=/usr/local
BASEDIR=$BASE/pd-admin2
FAULT=0
mycnfdef=${BASEDIR}/etc/my.cnf
for n in /etc/my.cnf /etc/mysql/my.cnf; do
   if [ -L $n ]; then
      Z=`./readlink $n`
      if [ $? -ne 0 ]; then
         FAULT=1
      fi
      if [ "$Z" != "${BASEDIR}/etc/my.cnf" ]; then
         echo "$n is a symlink but does not point to $mycnfdef"
         FAULT=1
      else 
         echo "(info: $n is a symlink which points to $mycnfdef)"
      fi
   elif [ -f $n ]; then
      echo "ERROR: Please rename $n!"
      FAULT=1
   fi
done

if [ $FAULT -ne 0 ]; then
   echo "exiting ..."
   exit 2
fi

if [ -L /home/mysql -o -e /home/mysql ]; then
   echo "Es existiert ein Dateiobjekt unter dem Namen /home/mysql"
   echo "Bitte benennen Sie es um oder entfernen Sie es, bevor Sie"
   echo "die Serverumgebung installieren."
   exit 1

fi
if [ `umask` != '0022' ]; then 
   echo "Warning: umask is set to `umask`, should be 0022."
   echo "Setting umask to 0022"
   umask 0022
   sleep 3 
fi

# erforderliche Komponenten installiert?
error=none
type -p wget 1>/dev/null 2>&1 || {
   echo "Kann wget nicht im Suchpfad finden." 
   error=error
}
type -p patch 1>/dev/null 2>&1|| {
   echo "Kann patch nicht im Suchpfad finden."
   error=error
}
set +e
DEBIAN=`cat /etc/debian_version 2>/dev/null`
set -e
if [ "$DEBIAN" == "3.1" ]; then
    GCCTEST="gcc-3.4"
    type -p gcc-3.4 1>/dev/null 2>&1|| {
       echo "Kann gcc-3.4 nicht im Suchpfad finden."
       echo "Bitte mit 'apt-get install gcc-3.4' nachinstallieren."
       echo
       error=error
    }
else
    GCCTEST="gcc"
    type -p gcc 1>/dev/null 2>&1|| {
       echo "Kann gcc nicht im Suchpfad finden."
       error=error
    }
fi
#
#  check architecture and glibc.i686 presence
#
if ! $HERE/tools/sed -e 's/a/b/' </dev/null 2>/dev/null; then
   echo "glibc fr i686 ist nicht installiert."
   error=error
   if type -p yum >/dev/null; then
      if yum -y install glibc.i686 >&/dev/null; then
         echo "glibc.i686 wurde mit <yum -y install glibc.i686> nachinstalliert."
         unset error
      else
         echo "glibc.i686 konnte mit <yum -y install glibc.i686> nicht nachinstalliert werden."
      fi
   fi
fi
type -p make 1>/dev/null 2>&1|| {
   echo "Kann make nicht im Suchpfad finden."
   error=error
}
type -p nroff 1>/dev/null 2>&1|| {
   echo "Kann nroff nicht im Suchpfad finden."
   error=error
}
type -p killall 1>/dev/null 2>&1|| {
   echo "Kann killall nicht im Suchpfad finden."
   error=error
}
type -p getopt 1>/dev/null 2>&1|| {
   echo "Kann getopt nicht im Suchpfad finden."
   error=error
}
if [ "$error" == "error" ]; then
   echo "Bitte installieren Sie die erforderlichen Komponenten nach."
   exit 1
fi 

# gcc funktionsfhig?
gcc tests/test.c -o tests/test > tests/test.log 2>&1 || {
   echo "Kann tests/test.c nicht kompilieren."
   echo "Bitte berprfen Sie die Installation des C-Compilers"
   exit
}

function is_running ()
{
   binary=$1
   killall -0 $binary >& /dev/null
}

# MySQL-Server gestartet?
{ is_running mysqld || is_running mariadbd; } && {
   echo "Es luft bereits eine MySQL-Instanz.  Diese muss vor "
   echo "Beginn der Installation beendet werden."
   exit 1
}

#
#  /etc/my.cnf shall be removed/renamed
#
if [ -f /etc/my.cnf ]; then
   echo " "
   echo "Die Datei /etc/my.cnf behindert die korrekte Installation"
   echo "der MySQL-Datenbank. Bitte benennen Sie die Datei um!"
   echo " "
   exit
fi
#
#  /root/.my.cnf
#
if [ -f /root/.my.cnf ]; then
   echo " "
   echo "Die Datei /root/.my.cnf behindert evtl. die korrekte Ausfhrung"
   echo "der MySQL-Programme. Bitte benennen Sie die Datei um!"
   echo " "
   exit
fi


# FTP-Port in Verwendung?
$HERE/tools/lsof -i TCP:21 1>/dev/null 2>&1 && {
    echo "Es hat sich bereits eine Anwendung an den FTP-Port (21) "
    echo "gebunden.  Bitte killen Sie diese."
    exit 1
}

# SMTP-Port in Verwendung?
$HERE/tools/lsof -i TCP:25 1>/dev/null 2>&1 && {
    echo "Es hat sich bereits eine Anwendung an den SMTP-Port (25) "
    echo "gebunden.  Bitte killen Sie diese."
    exit 1
}

# POP3-Port in Verwendung?
$HERE/tools/lsof -i TCP:110 1>/dev/null 2>&1 && {
    echo "Es hat sich bereits eine Anwendung an den POP3-Port (110) "
    echo "gebunden.  Bitte killen Sie diese."
    exit 1
}

# MySQL-Port in Verwendung?
$HERE/tools/lsof -i TCP:3306 1>/dev/null 2>&1 && {
    echo "Es hat sich bereits eine Anwendung an den MySQL-Port (3306) "
    echo "gebunden.  Bitte killen Sie diese."
    exit 1
}

process_command_line "$@"

if unattended_install; then
   :
else
   echo "Erkennen Sie die Providerdienste.de Lizenzbedingungen"
   myprompt 'jnN' "fr den Betrieb der Software pd-Admin an (j/n)" "N"

   if [ "$answer" != "j" ]
   then
      exit
   fi
fi

#
#	Benutzer/Gruppen anlegen
#

mkdir -p /opt/pdadmin

grep "^pdadmin:" /etc/group >/dev/null 2>&1 || {
   groupadd pdadmin
}

grep "^pdadmin:" /etc/passwd >/dev/null 2>&1 || {
   useradd pdadmin -g pdadmin -G '' -s /bin/false -d /opt/pdadmin/tmp
}

grep "^nogroup:" /etc/group >/dev/null 2>&1 || {
   groupadd nogroup
}

grep "^www:" /etc/group >/dev/null 2>&1 || {
   groupadd www
}
grep "^www:" /etc/passwd >/dev/null 2>&1 || {
   useradd -g www -G '' www -d /tmp
}

grep "^daemon:" /etc/group >/dev/null 2>&1 || {
   groupadd daemon
}

grep "^mysql:" /etc/passwd >/dev/null 2>&1 || {
   useradd -g daemon -G '' mysql -d /tmp
}

grep "^pdbackup:" /etc/passwd >/dev/null 2>&1 || {
   useradd pdbackup -G '' -s /bin/false -d /tmp
}


echo ""
if unattended_install; then
   answer=j
else
   myprompt 'JjnN' "Mchten Sie die Server-Umgebung (SE) installieren? (j/n)" "j"
fi

if [ "$answer" == "j" -o "$answer" == "J" ]; then
   STDENV=true
   PATH=${SEINSTDIR}/bin:$PATH
#
#  prefetch files needed for qmail2.sh
#
   . ./prefetch.sh
   download_std_serv_env 

   chown root:www ${SEINSTDIR}/logs
   chmod 0750 ${SEINSTDIR}/logs
   hash -r
   cgi=${SEINSTDIR}/htdocs
   echo
   #
   #  MYSQLDATADIR nach /home/mysql
   #
   mv ${SEINSTDIR}/var/mysql /home/
   ln -s /home/mysql ${SEINSTDIR}/var/
   #
   #	mysql anfahren
   #
   echo "MySQL wird angefahren ..."
if [ "$SERIES" = "8" ]; then
   cat >> $SEINSTDIR/etc/my.cnf <<_
default_authentication_plugin=mysql_native_password
_
fi
   ${SEINSTDIR}/bin/mysqld_safe > /tmp/MySQL.log 2>&1 &
   sleep 6
   # 
   #  testen, ob prozess vorhanden, sonst abbruch
   #
   { is_running mysqld || is_running mariadbd; } || {
      echo "MySQL konnte nicht gestartet werden."
      echo "mgliche Fehlerhinweise (/tmp/MySQL.log):"
      echo " "
      cat /tmp/MySQL.log
      echo " "
      MYERRORFILE=${SEINSTDIR}/var/mysql/$HOSTNAME.err
      if [ ! -f $MYERRORFILE ]; then
         echo "$MYERRORFILE nicht gefunden"
      else 
         echo "gebe $MYERRORFILE aus:"
         cat $MYERRORFILE
      fi
      echo " "
      exit 1
   }

   set +e
   . tools/configure-apache.inc

function ask_for_root_email_address()
{
#  ask for root/postmaster email
#  2005-01-11 stvo

if [ "$POSTMASTER_ROOT_ADDRESS" == "" ]; then
   POSTMASTER_ROOT_ADDRESS=""
   POSTMASTER_ROOT_ADDRESS2=" "
fi

until [ "$POSTMASTER_ROOT_ADDRESS" == "$POSTMASTER_ROOT_ADDRESS2" ]; do

echo " "
echo "Bitte geben Sie eine E-mail-Adresse an, an die alle Nachrichten"
echo "fr den Postmaster und den Superuser (root) zugestellt werden"
echo "sollen."
echo " "

echo -n "Eingabe     : "
read POSTMASTER_ROOT_ADDRESS
echo -n "Wiederholung: "
read POSTMASTER_ROOT_ADDRESS2

done
}

if unattended_install; then
   POSTMASTER_ROOT_ADDRESS=$email
else
   ask_for_root_email_address
fi

#
#  
#
   . qmail2.sh | tee qmail2.log

if [ ${PIPESTATUS[0]} -ne 0 ]; then
   echo qmail2.sh failed
   exit 1
fi

echo "AFTER QMAIL2"
touch /var/qmail/control/virtualdomains
touch /var/qmail/control/taps
cat - <<_ >/var/qmail/alias/.qmail-srs-default
|/opt/pdadmin/bin/srs_bounce.pl
_


#
#  clamav initialisieren
#
      echo "clamav wird initialisiert ..."
      ${SEINSTDIR}/CONFIGURE/configure-cl.sh
#
#  2005-03-29 stvo
#  prepare simscan
#
echo "configuring simscan ..."
${SEINSTDIR}/CONFIGURE/configure-sc.sh


echo $POSTMASTER_ROOT_ADDRESS > /var/qmail/alias/.qmail-root
echo $POSTMASTER_ROOT_ADDRESS > /var/qmail/alias/.qmail-postmaster


   if test -L /usr/bin/perl; then
   	rm -f /usr/bin/perl
   fi
   if test -e /usr/bin/perl; then
  	mv /usr/bin/perl /usr/bin/perl__
   fi
   ln -fs ${SEINSTDIR}/bin/perl /usr/bin/perl

   if test -L /usr/local/bin/perl; then
  	rm -f /usr/local/bin/perl
   fi
   if test -e /usr/local/bin/perl; then
	mv /usr/local/bin/perl /usr/local/bin/perl__
   fi
   ln -fs ${SEINSTDIR}/bin/perl /usr/local/bin/perl

   if test -e /etc/debian_version; then
        ln -s /usr/share/perl5/Debconf ${SEINSTDIR}/lib/perl5/site_perl
        ln -s /usr/share/perl5/Debian ${SEINSTDIR}/lib/perl5/site_perl 
        ln -s /usr/share/perl5/DebianNet.pm ${SEINSTDIR}/lib/perl5/site_perl
   fi

else
   echo " "
   echo "Zur Installation von pd-admin ohne die Server-Umgebung (SE)" 
   echo "fhren Sie bitte das Skript"
   echo " "
   echo "    install.sh"
   echo " "
   echo "aus."
   exit 0
fi



# 2003-12-19 stvo: in 0.007 enthalten
# perl -e "install Crypt::PasswdMD5" -MCPAN
# perl -e "install Unix::Syslog" -MCPAN

VADMINPW=`$HERE/tools/randompw.pl`
echo . > /var/qmail/users/assign

if [ "$VADMINPW" == "" ]; then
   echo "Fataler Fehler:  Konnte Passwort fuer die vadmin-"
   echo "Datenbank nicht generieren."
   exit 1
fi
#
#  prepare checkuser-patch config file
#
rm -f /var/qmail/vadmin-check.pl
touch /var/qmail/vadmin-check.pl
chown root:root /var/qmail/vadmin-check.pl
chmod u=rw,go= /var/qmail/vadmin-check.pl
cat <<_ >>/var/qmail/vadmin-check.pl
\$database         = "vadmin";
\$hostname         = "localhost";
\$user             = "vadmin";
\$password         = "$VADMINPW";
_
chown qmaild:qmail /var/qmail/vadmin-check.pl

#
# checksmtppasswd konfigurieren ##
#
   $HERE/tools/sed -es/#VADMINPW#/$VADMINPW/ $HERE/templates/checksmtppasswd-template \
      > /bin/checksmtppasswd

chmod 700 /bin/checksmtppasswd
chown qmaild /bin/checksmtppasswd
ln -sf /bin/checksmtppasswd ${SEINSTDIR}/bin/checksmtppasswd 

grep "^popuser:" /etc/group >/dev/null 2>&1 || {
        echo "Lege Gruppe popuser an."
        groupadd popuser
}

grep "^popuser:" /etc/passwd >/dev/null 2>&1 || {
        echo "Lege Benutzer popuser an."
        useradd -s /bin/false -g popuser -G '' -d /home/popuser popuser
}

#useradd -s /bin/false -d /home/popuser popuser

if [ ! -d /home/popuser ]
then
   echo "Lege Verzeichnis /home/popuser an."
   mkdir -p /home/popuser
   echo "99" > /home/popuser/spamfilter_exitcode
   chown -R popuser:popuser /home/popuser
   chmod 750 /home/popuser
fi

if [ ! -d /home/popuser/popboxen ]
then
  echo "Lege Verzeichnis /home/popuser/popboxen an."
  mkdir -p /home/popuser/popboxen
  chown popuser:popuser /home/popuser/popboxen
  chmod 750 /home/popuser/popboxen
fi

cd $HERE

mkdir -p /opt/pdadmin
chattr -i /opt/pdadmin/lib/*.so >/dev/null 2>&1
cp -a ./pdadmin /opt/

#chown root:www /opt/pdadmin/www/customer/customer.cgi /opt/pdadmin/www/customer/navigation.cgi /opt/pdadmin/www/administrator/navigation.cgi /opt/pdadmin/www/administrator/administrator.cgi 
chown root:www /opt/pdadmin/www/customer/navigation.cgi /opt/pdadmin/www/administrator/navigation.cgi /opt/pdadmin/www/customer/customer.cgi /opt/pdadmin/www/administrator/administrator.cgi /opt/pdadmin/www/pop3/pop3.cgi /opt/pdadmin/www/pop3/navigation.cgi /opt/pdadmin/www/pop3/autoconfigure.cgi

chown popuser:popuser /opt/pdadmin/bin/forward_filter*
chmod 4755 /opt/pdadmin/bin/forward_filter

chmod 4750 /opt/pdadmin/www/customer/customer.cgi
chmod 4750 /opt/pdadmin/www/customer/navigation.cgi
chmod 4750 /opt/pdadmin/www/administrator/navigation.cgi
chmod 4750 /opt/pdadmin/www/administrator/administrator.cgi
chmod 4750 /opt/pdadmin/www/pop3/pop3.cgi
chmod 4750 /opt/pdadmin/www/pop3/autoconfigure.cgi
chmod 4750 /opt/pdadmin/www/pop3/navigation.cgi

#
#  Ticket 1210338
#
mkdir -p /opt/pdadmin/etc/ssl-validation/.well-known/acme-challenge
chown pdadmin /opt/pdadmin/etc/ssl-validation/.well-known
chown pdadmin /opt/pdadmin/etc/ssl-validation/.well-known/acme-challenge

cd $mypath

### angepasste Konfiguration bei Server-Umgebung verwenden
if [ "$STDENV" != "" ]; then
  cp templates/config.pl-dist /opt/pdadmin/etc/pdadmin.conf
  $HERE/tools/sed \
      -es/#SERVERNAME#/$SERVERNAME/g \
      -es/#IPADDRESS#/$IPADD/g \
      /opt/pdadmin/etc/pdadmin.conf > /opt/pdadmin/etc/pdadmin.conf~
  grep -q url_mapping /opt/pdadmin/etc/pdadmin.conf~ && mv /opt/pdadmin/etc/pdadmin.conf~ /opt/pdadmin/etc/pdadmin.conf
fi
### /

#mkdir -p /opt/pdadmin/lib
#chattr -i /opt/pdadmin/lib/*.so >/dev/null 2>&1
##cp lib/*.so* /opt/pdadmin/lib
chmod 444 /opt/pdadmin/lib/*.so
chattr +i /opt/pdadmin/lib/*.so >/dev/null 2>&1

#
#  2010-10-01 stvo
#  srs config
#
ranstr=`$HERE/tools/randompw.pl`
cat - <<_ >/opt/pdadmin/etc/srs.conf
$SERVERNAME
$ranstr
_

echo
echo
while [ "$cgi" = "" ] || [ ! -d "$cgi" ]
do
    $echo "Wohin sollen die Webinterfaces installiert werden? ${nnl}"
    read cgi
done

cd $mypath

echo -n "Teste Verbindung zur MySQL-Datenbank..."
${SEINSTDIR}/bin/mysqlshow -uroot --password="" mysql >/dev/null 2>&1
if [ $? -eq 0 ]; then

   echo "Erfolgreich."
   echo ""

   MYSQLROOTPW=`$HERE/tools/randompw.pl`
   if [ "$MYSQLROOTPW" == "" ]; then
      echo "Fataler Fehler:  Konnte root-Passwort fuer die MySQL-"
      echo "Datenbank nicht generieren."
      exit 1
   fi

   echo "Setze root-Passwort fuer MySQL-Datenbankserver auf '$MYSQLROOTPW'."
   echo ""

###. ./clean-user-table.sql
if ! ${user_table_is_updatable[$SERIES]}; then
   ${SEINSTDIR}/bin/mysql -u root --password="" mysql <<EOT
-- set password for 'root'@'localhost' = '$MYSQLROOTPW';
alter user 'root'@'localhost' identified by '$MYSQLROOTPW';
DELETE FROM user WHERE User='';
DELETE FROM db WHERE User='';
FLUSH PRIVILEGES;
EOT
elif [ "$SERIES" = "8" ]; then
   ${SEINSTDIR}/bin/mysql -u root --password="" mysql <<EOT
set password for 'root'@'localhost' = '$MYSQLROOTPW';
alter user 'root'@'localhost' identified with mysql_native_password by '$MYSQLROOTPW';
DELETE FROM user WHERE User='';
DELETE FROM db WHERE User='';
FLUSH PRIVILEGES;
EOT
else
   ${SEINSTDIR}/bin/mysql -u root --password="" mysql <<EOT
UPDATE user SET ${mysql_credential_column_name[$SERIES]}=PASSWORD('$MYSQLROOTPW') WHERE User='root';
DELETE FROM user WHERE User='';
DELETE FROM db WHERE User='';
FLUSH PRIVILEGES;
EOT
fi
if [ $? -ne 0 ]; then
   echo "Passworte konnten nicht gesetzt werden"
   exit 97
fi

else
   echo
   echo "Bitte geben Sie das root-Passwort fr Ihren MySQL-Datenbankserver ein."
   echo "Dieses wird zur Installation der pd-Admin 2.0 Datenbank bentigt."
   echo

   pp 'MySQL root password'
   MYSQLROOTPW=$PASSWORD
fi



echo "Speichere MySQL root-Passwort in /opt/pdadmin/etc/mysql_rootpw.conf"
echo $MYSQLROOTPW > /opt/pdadmin/etc/mysql_rootpw.conf
chmod 600 /opt/pdadmin/etc/mysql_rootpw.conf

#echo
#echo "Bitte geben Sie ein Passwort fuer den MySQL-Benutzer vadmin ein."
#echo "Unter dieser Berechtigung wird in Zukunft die Verwaltung der DB erledigt."
#echo
#pp 'vadmin password'

echo "Setze Passwort fuer Datenbank-Benutzer vadmin: '$VADMINPW'"
echo

echo "create database vadmin;" | ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW 
if [ $? -ne 0 ]; then
   echo "Datenbank konnte nicht angelegt werden."
   exit 98
fi

if [ "$SERIES" = "8" -o "$SERIES" = "9" ]; then
   ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW mysql <<_
CREATE USER 'vadmin'@'localhost' IDENTIFIED BY '$VADMINPW';
GRANT ALL PRIVILEGES on vadmin.* to 'vadmin'@'localhost';
_
else
   echo "grant all privileges on vadmin.* to vadmin@localhost identified by '$VADMINPW';" | ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW 
   ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW mysql <<XXX
UPDATE user SET ${mysql_credential_column_name[$SERIES]}=PASSWORD('$VADMINPW') WHERE User='vadmin';
FLUSH PRIVILEGES;
XXX
fi


${SEINSTDIR}/bin/mysql -u vadmin -p$VADMINPW vadmin < $HERE/templates/mysql_dump.txt

##


TIME=`date +%s`
(crontab -l 2>/dev/null; cat templates/cron.txt) > /tmp/crontab.$TIME
grep -v ^# /tmp/crontab.$TIME > /tmp/crontab.$TIME~
crontab /tmp/crontab.$TIME~
rm /tmp/crontab.$TIME
rm /tmp/crontab.$TIME~
type -p rrdtool >/dev/null 2>&1 && ( crontab -l | grep -q /opt/pdadmin/bin/stats_load.sh || echo "*/5  *   * * * /opt/pdadmin/bin/stats_load.sh update" | bash tools/crontab_add.sh )

##
useradd -G '' codomain
mkdir -p /home/codomain
chown codomain:root /home/codomain
chmod 751 /home/codomain
#sed "s/PASSWORD/$CODOPASS/" codomain.php > /home/codomain/codomain.php
#echo "grant select on vadmin.codomains to codomain@localhost identified by '$CODOPASS';" | ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW
#echo "grant select on vadmin.domains to codomain@localhost;" | ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW

touch /var/qmail/users/poppasswd
echo . >> /var/qmail/users/assign

echo "Kopiere Passwortcheck"
cp popuser-utils/checkpoppasswd /bin 
chmod 700 /bin/checkpoppasswd

echo "Kopiere Mailquotacheck"
cp popuser-utils/mailquotacheck.sh /home/popuser
chmod 700 /home/popuser/mailquotacheck.sh
chown popuser:popuser /home/popuser/mailquotacheck.sh

touch /home/popuser/spamfilter
chown popuser:popuser /home/popuser/spamfilter

#
#  2004-12-15
#
echo "Installiere PDPASSWD.pm nach site_perl"
cp -a PDPASSWD.pm ${SEINSTDIR}/lib/perl5/site_perl/

grep '/bin/false' /etc/shells 1>/dev/null 2>&1 || echo "/bin/false" >> /etc/shells

$HERE/tools/sed "s/yourpassword/$VADMINPW/" /opt/pdadmin/etc/pdadmin.conf > /opt/pdadmin/etc/pdadmin.conf~
mv /opt/pdadmin/etc/pdadmin.conf~ /opt/pdadmin/etc/pdadmin.conf

chmod 600 /opt/pdadmin/etc/pdadmin.conf

## proftpd konfigurieren ##
echo "proftpd wird eingerichtet ..."
${SEINSTDIR}/CONFIGURE/configure-pf.pl
svc -d /service/proftpd
sleep 1
svc -u /service/proftpd
## proftpd               ##

./install_auto_spam.sh 2>/dev/null

echo "Dovecot 2.2 wird vorbereitet ..."
${SEINSTDIR}/CONFIGURE/dovecot-2.2.sh
echo "Fertig."
echo " "

echo "RoundCube Webmail wird eingerichtet ..."
${SEINSTDIR}/CONFIGURE/roundcube.pl
echo "Fertig."
echo " "

DOTLESSVERSION=${SEVERSION:0:1}${SEVERSION:2:3}
if [ $DOTLESSVERSION -ge 0325 ]; then
   case $SERIES in
      1 | 2 | 3)
         echo "phpMyAdmin and phpPgAdmin no longer available, use \$hostname/adminer"
         ;;
      *)
         echo "phpMyAdmin < 4.8.4 and phpPgAdmin no longer available, use \$hostname/adminer"
         if [ ! -f ${SEINSTDIR}/htdocs/phpMyAdmin/config.inc.php ]; then
            echo "phpMyAdmin 4.8.4: initializing config.ini.php: blowfish_secret"
            touch ${SEINSTDIR}/htdocs/phpMyAdmin/config.inc.php
            chown root:phpmyadm ${SEINSTDIR}/htdocs/phpMyAdmin/config.inc.php
            chmod 0640 ${SEINSTDIR}/htdocs/phpMyAdmin/config.inc.php
            blowfish_secret=$(dd if=/dev/urandom bs=24 count=1 2>/dev/null | base64 2>/dev/null)
            cat > ${SEINSTDIR}/htdocs/phpMyAdmin/config.inc.php <<_
<?php
// auto generated by se-update.sh
\$cfg['blowfish_secret'] = '$blowfish_secret';
_
         fi
   esac
fi

echo "Installation abgeschlossen"
echo

echo "MySQL-Datenbank wird neu gestartet..."
${SEINSTDIR}/bin/mysqladmin -p`cat /opt/pdadmin/etc/mysql_rootpw.conf` shutdown
echo "Fertig."
echo " "

### ./tools/prepare_mysql_for_ci.pl

#/usr/local/pd-admin2/CONFIGURE/configure-ci.pl
/usr/local/pd-admin2/CONFIGURE/configure-sm.sh

mkdir -p /opt/pdadmin/tmp
chown pdadmin /opt/pdadmin/tmp
chmod 700 /opt/pdadmin/tmp

mkdir -p /opt/pdadmin/sslcerts
chown pdadmin /opt/pdadmin/sslcerts
chmod 700 /opt/pdadmin/sslcerts

mkdir -p /opt/pdadmin/nav
chown www /opt/pdadmin/nav
chmod 700 /opt/pdadmin/nav

chgrp pdadmin /usr/local/pd-admin2/UPDATE.INF/
chmod 750 /usr/local/pd-admin2/UPDATE.INF/

mv /opt/pdadmin/etc/awstats.conf.master.dist /opt/pdadmin/etc/awstats.conf.master
mv /opt/pdadmin/etc/webalizer.conf.master.dist /opt/pdadmin/etc/webalizer.conf.master
mv /opt/pdadmin/etc/customer_menu_individual.conf.example customer_menu_individual.conf

if [ -d /usr/local/pd-admin2/awstats ]; then
   chmod 755 /usr/local/pd-admin2/awstats
fi

mkdir -p /home/simscan/.spamassassin
chown spamd /home/simscan/.spamassassin
chmod 750 /home/simscan/.spamassassin

chmod 755 /opt/pdadmin/bin/backup_write_dump.sh
chmod 755 /opt/pdadmin/bin/backup_user.sh

mkdir -p /opt/pdadmin/greylist/whitelist
mkdir -p /opt/pdadmin/greylist/database

chown qmaild /opt/pdadmin/greylist/whitelist /opt/pdadmin/greylist/database

chown root:root /opt/pdadmin/etc/*

chown root:www /opt/pdadmin/www/pop3/*.cgi
chmod u+s /opt/pdadmin/www/pop3/*.cgi

#
#  2010-05-25 enable greylisting
#
grep -q greylist /etc/tcp.smtp 1>/dev/null || cp $HERE/templates/tcp.smtp /etc/
test -e /var/qmail/control/greylist || cp $HERE/templates/control /var/qmail/control/greylist
tcprules /etc/tcp.smtp.cdb /etc/tcp.smtp.tmp < /etc/tcp.smtp

#
#  2012-04-16
#
if [ -e /opt/pdadmin/etc/webdav.conf.dist ] && [ ! -e /opt/pdadmin/etc/webdav.conf ]; then
   mv /opt/pdadmin/etc/webdav.conf.dist /opt/pdadmin/etc/webdav.conf
fi

echo "register rssh as shell"
grep -q rssh /etc/shells || echo ${SEINSTDIR}/bin/rssh >> /etc/shells
mv /opt/pdadmin/etc/shells.conf.dist /opt/pdadmin/etc/shells.conf

if grep 'Ubuntu 16.04' /etc/os-release &>/dev/null; then
   sed -i 's-\(.*::.*\)-###\1-g' /etc/hosts
fi


if test -f "/var/qmail/bin/dk-filter"; then
   ${BASEDIR}/CONFIGURE/configure-dkim.sh -er
   echo "| "
   echo "| DKIM activated"
   echo "| "
fi

mkdir -p /var/qmail/users/
chown root:qmail /var/qmail/users/
chmod 750 /var/qmail/users/

/opt/pdadmin/bin/get_license.sh

echo Waiting for MySQL
sleep 10

configs=(
   "insert into config set varvalue='8.2.99', varname='php5_version';"
   "insert into config set varvalue='AP24', varname='webserver';"
   "insert into config set varvalue='1', varname='letsencrypt';"
)

for c in "${configs[@]}"; do
   ${SEINSTDIR}/bin/mysql -u root -p$MYSQLROOTPW vadmin -e "$c"
done

test -e /lib/libexpat.so.1 || ln -s /opt/pdadmin/lib/libexpat.so.1 /lib

test -e BUILD_x86_64 && {
   for i in libssl libcrypto; do 
      test -e /lib64/$i.so.10 || ln -s /usr/local/pd-admin2/lib/$i.so.1.0.0 /lib64/$i.so.10
      test -e /usr/lib/x86_64-linux-gnu/$i.so.1.1 && test -e /usr/lib/x86_64-linux-gnu/$i.so.10 || \
           ln -s /usr/lib/x86_64-linux-gnu/$i.so.1.1 /usr/lib/x86_64-linux-gnu/$i.so.10
   done
}

if [ "$SERIES" -ne "9" ]; then
   mysql -u root -p`cat /opt/pdadmin/etc/mysql_rootpw.conf` vadmin -e "update mysql.user set Create_tmp_table_priv='Y' where user='vadmin';"
   mysql -u root -p`cat /opt/pdadmin/etc/mysql_rootpw.conf` vadmin -e "flush privileges;"
fi

/opt/pdadmin/bin/httpd_vhosts.pl > /dev/null 2>&1
/opt/pdadmin/bin/letsencrypt `/opt/pdadmin/bin/hostname.pl`
/opt/pdadmin/bin/update_host_certificate.sh


echo "pd-admin wurde erfolgreich installiert."
echo
echo "Bitte fhren Sie nun das Skript /opt/pdadmin/bin/create_reseller.pl"
echo "aus, um einen Administrator-Account anzulegen."
