#!/usr/local/pd-admin2/bin/perl -w

use strict;
use DBI;

#####
##### Read Configuration
#####

use vars qw($popuser_name $dsn $user $password $domainfile $apacherestart %mysql);

do "/opt/pdadmin/etc/pdadmin.conf" or die "Can't read configuration: $!\n";
do "./mysql.structure.pl" or die "Can't read mysql structure: $!\n";

my $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
$SIG{__DIE__} = sub {  $dbh->disconnect if $dbh;  die @_;};

####
####  subs
####


###############################
sub get_row(@) {
###############################

  my $query = shift;

  my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
  my $rv;
  unless ($rv = $sth->execute(@_)) {
    $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
    $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
    $rv  = $sth->execute or die "cannot execute query '$query': ". $dbh->errstr;
  }
  my @row = $sth->fetchrow_array;
  $sth->finish;
  return @row;

}

sub update_row {
  my $query = shift;
  my $rueck = '';
  my ($sth, $rv, $rc);
  if ($sth = $dbh->prepare($query)) {
    if ($sth->execute) {
      $rueck = $sth->{mysql_insertid} || 1;
    }
    $rc = $sth->finish;
  }
  return $rueck;
}

sub field_definition($$) {
  my $table = shift;
  my $field = shift;
  my $def   = "`$field` $mysql{$table}{$field}{type}";
  my $pkey;
  if (lc $mysql{$table}{$field}{null} ne 'yes') {
    $def .= " NOT NULL";
  }

  if (defined $mysql{$table}{$field}{default}) {
     $def .= " default '$mysql{$table}{$field}{default}'"; 
  }

  if (defined $mysql{$table}{$field}{key} and length $mysql{$table}{$field}{key}) {
    if (lc $mysql{$table}{$field}{key} eq 'pri') {
       $pkey = $field;
    }
  }

  if (defined $mysql{$table}{$field}{extra} and length $mysql{$table}{$field}{extra}) {
     $def .= " $mysql{$table}{$field}{extra}";
  }

  return $def

}

sub create_table($) {
   my $table = shift;

   my $def = "CREATE TABLE `$table` (\n";

   my $pkey; my @keys;
   my @fields;  
   for my $field (keys %{$mysql{$table}}) {

      $field = lc $field;

      push @fields, "  ". field_definition $table, $field;
      if (defined $mysql{$table}{$field}{key} and length $mysql{$table}{$field}{key}) {
         if (lc $mysql{$table}{$field}{key} eq 'pri') {
            $pkey = $field;
         }
         if  (lc $mysql{$table}{$field}{key} eq 'mul') {
           push @keys, $field;
         }
      }
   }

   if (defined $pkey and $pkey) {
     push @fields, "  PRIMARY KEY ($pkey)";
   }
   for my $key (@keys) {
     push @fields, "  KEY $key ($key)";
   }

   $def .= join ",\n", @fields;
   $def .= "\n);";
 
   return $def

}


####
####   main
####

my ($mysql_version)= get_row 'SELECT @@version';
($mysql_version) = $mysql_version =~ /^(\d+\.\d+)/;

if ($mysql_version > 5.5) {
   update_row "SET SQL_MODE='ALLOW_INVALID_DATES'";
}

my $query = "show tables;";
my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
my $rv  = $sth->execute or die "cannot execute query '$query'";

my %table;
while (my ($table) = $sth->fetchrow_array) {
   $table{$table} = 1;
}
$sth->finish;

for my $table (sort keys %mysql) { 

  unless (defined $table{$table} and $table{$table}) {
    print "Erstelle Tabelle $table.\n";
    my $query = create_table($table);
    update_row $query or die "Kann Tabelle $table nicht erstellen.\n";
  } else {

     my $query1 = "describe `$table`;";
     my $sth1   = $dbh->prepare($query1) or die "cannot prepare query '$query1'";
     my $rv1    = $sth1->execute or die "cannot execute query '$query1'";
     my %otable;
     while (my ($field, $type, $null, $key, $default, $extra) = $sth1->fetchrow_array) {

       $field = lc $field; 

       $otable{$table}{$field}{type} = $type;
       $otable{$table}{$field}{null} = $null;
       $otable{$table}{$field}{key} = $key;
       $otable{$table}{$field}{default} = $default;
       $otable{$table}{$field}{extra} = $extra;
     }
     $sth1->finish;

     for my $field (keys %{$mysql{$table}}) { 

        $field = lc $field;

        unless (defined $otable{$table}{$field}{type}) {
          my $def = field_definition $table, $field;
          print "Lege Feld $table.$field an.\n";
          my $query = "alter table `$table` add ". $def. ";";
          update_row $query or die "Kann Feld $table.$field nicht anlegen.\n";
          next;
        }

        my $changed = 0;
        for my $kat (keys %{$mysql{$table}{$field}}) {

            unless (defined $otable{$table}{$field}{$kat} == 
                    defined $mysql{$table}{$field}{$kat}) {
               $changed = 1;
               last;
            }

            if (defined $otable{$table}{$field}{$kat}) {
              unless ($otable{$table}{$field}{$kat} eq $mysql{$table}{$field}{$kat}) {
                 $changed = 1;
                 last;
              }
            }
        }

        if ($changed) { 
          my $def = field_definition $table, $field;
          print "Aendere Feld $table.$field\n";
          my $query = "alter table `$table` change `$field` ". $def. ";";
          update_row $query or die "Kann Feld $table.$field nicht aendern.\n";
        }

     }

  }

}
