#!/usr/local/pd-admin2/bin/perl -w

###
###  pd-admin - httpd_log.pl
###
###  (c) 1999 - 2004 by Daniel Bradler
###
###  $Id$
###

use strict;
use DBI;
use Socket;

###
###  Read Configuration
###

our ($dsn, $user, $password, $backup_dir, $backup_rotation, $backup_user_exclude, $httpd_conf_template);

do "/opt/pdadmin/etc/pdadmin.conf" or die "Can't read configuration: $!";
my $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
$SIG{__DIE__} = sub {  $dbh->disconnect if $dbh;  die @_;};

###
###  Subroutines
###

#####################
sub check_home($) {
#####################
   my $home = shift;
   ($home)  = $home =~ m#^(/[\w/.-]+)$#;
   return $home;
}

#######################
sub get_home($) {
#######################
   my $user = shift;
   my $home = (getpwnam($user))[7];
   $home    = check_home($home);
   return $home;
}

#######################
sub get_row($) {
#######################
  my $query = shift;
  my $sth   = $dbh->prepare($query) or die "cannot prepare query '$query'\n";
  my $rv    = $sth->execute or die "cannot execute query '$query'";
  my @row   = $sth->fetchrow_array;
  $sth->finish;
  return @row;
}

#######################
sub get_databases($) {
#######################
  my $customerid = shift; 
  my $query      = "select name from mysqldb where owner='$customerid'";
  my $sth        = $dbh->prepare($query) or die "cannot prepare query '$query'\n";
  my $rv         = $sth->execute or die "cannot execute query '$query'";
  my @databases;
  while (my ($database) = $sth->fetchrow_array) {
     push @databases, $database;
  } 
  return @databases
}

###
###  Main
###

unless (defined $backup_dir and length $backup_dir) {
  die "Please set \$backup_dir in /opt/pdadmin/etc/pdadmin.conf\n";
}

$backup_user_exclude ||= '';
$backup_rotation     ||= 1;
my $rotation_system    = (int time / 86400) % $backup_rotation;

my @system_files_to_backup_candidates = (
   $httpd_conf_template,
   '/usr/local/pd-admin2/httpd-2.4/conf/httpd24.conf-template'
);

my @system_files_to_backup = 
   grep { -e $_ }
   @system_files_to_backup_candidates;

system ('/opt/pdadmin/bin/backup_system.sh', $backup_dir, $rotation_system, @system_files_to_backup);

unless (defined $backup_dir and length $backup_dir) {
   print 'Backup directory not configured.', "\n";
   print 'Please set $backup_dir in /opt/pdadmin/etc/pdadmin.conf', "\n";
   exit 
}

my $query = "select t1.id, t1.login, t2.backup from users as t1, accounts as t2 ".
            "where t2.backup > 0 and t1.account=t2.id";
my $sth   = $dbh->prepare($query) or die "cannot prepare query '$query'";
my $rv    = $sth->execute or die "cannot execute query '$query'";

while (my ($customerid, $customer, $backup_days)  = $sth->fetchrow_array) {

   my $home     = get_home $customer;
   my $rotation = (int time / 86400) % $backup_days;

   next unless (defined $home);
   next unless (defined $customerid and $customerid =~ /^\d+$/);
   next unless (defined $customer   and $customer   =~ /^[a-z0-9]+$/);
 
   my @databases = get_databases $customerid;
   while (defined (my $database = shift @databases)) {
       next unless ($database =~ /^\w+$/);
       system "/opt/pdadmin/bin/backup_mysql.sh $customer $home $database";
   }

   system "/opt/pdadmin/bin/backup_user.sh $customer $home $backup_dir $rotation $backup_user_exclude";

}

$sth->finish;
