#!/bin/bash

set -CEeuo pipefail

nuldec=/usr/local/pd-admin2/bin/nuldec
$nuldec <<<'' >/dev/null
type lz4 >/dev/null

function report_errors_and_cleanup_tmpdirs
{
   cat $log_tmpdir/TAR.STDOUT
   exec 1<&-
   cat $log_tmpdir/TAR.STDERR >&2

   rm -fr "$tmpdir" 2>/dev/null || true
   rm -fr "$log_tmpdir" 2>/dev/null || true
}

login=$1

log_tmpdir=$(mktemp -d "/tmp/${login}.import.XXXXXX")
tmpdir=$(mktemp -d "/home/${login}.import.XXXXXX")
trap "report_errors_and_cleanup_tmpdirs" EXIT

(
   cd "$tmpdir"
   chown "$login" .
   $nuldec | setuidgid "$login" tar -Ilz4 -x || exit $?
   if [[ ! -d ./$login ]]; then
      echo "directory $login is missing (empty stream from server?)" >&2
      false
   fi
   if [[ -d ./$login/logs ]]; then
      chown root "./$login/logs" || true
   fi

   mkdir -p "/home/errorlogs/$login"
   setuidgid "$login" mkdir -p "../$login/logs/errorlogs"
   mount --bind "/home/errorlogs/$login" "../$login/logs/errorlogs"

   test -e "./$login/logs/errorlogs" && {
      mv "./$login/logs/errorlogs" .
      cp -bfaxl "./errorlogs/." "/home/errorlogs/$login/."
      chown -hR "root:$login" "/home/errorlogs/$login"
      chown root:www "/home/errorlogs/$login"
   }

   cp -bfaxl "./$login/." "../$login/."
   chgrp www "/home/$login"
   cd ..
) >$log_tmpdir/TAR.STDOUT 2>$log_tmpdir/TAR.STDERR
