#!/bin/bash
#
#  import-customer.sh
#  Ticket 1524227
#  2021-04-13 stvo
#
set -CEeuo pipefail
#set -x
nousage=

type lz4 >&/dev/null || { echo "lz4 is missing on the client" && false; }

if [[ ${JOB_ID-} ]]; then
   echo $JOB_ID
   logdir="/tmp/import-customer.$JOB_ID"
   mkdir "$logdir"
   chmod o= "$logdir"
   stdout="$logdir/STDOUT"
   stderr="$logdir/STDERR"
   exec 2<> $stderr
   exec 1<> $stdout
fi

usage ()
{
   echo "(import-customer.sh:$BASH_LINENO)" >&2
   if (($nousage)); then
      return
   fi
   echo "$0 [options]

Options

--remote-server            hostname (src)
--remote-customer-login    login (src)
--remote-customer-password password (src)
--reseller                 login (dst)
--account                  account (dst)
--preflight                only validate credentials
--stop-services            disable all imported services
--stop-cronjobs            disable all imported cronjobs
--trace                    trace processing
"
}

trap usage ERR

account=()
upstream_preflight=()
downstream_preflight=()
upstream_lang=()
downstream_lang=()
stop_services=()

while (($#)); do
   case $1 in
      --remote-server)
         remote_server=$2
         shift 2
         ;;
      --remote-customer-login)
         remote_customer_login=$2
         shift 2
         ;;
      --remote-customer-password)
         remote_customer_password=$2
         shift 2
         ;;
      --reseller)
         reseller=$2
         shift 2
         ;;
      --account|--plan)
         plan=$2
         shift 2
         account=(--account "$plan")
         ;;
      --preflight)
         upstream_preflight=(--data-urlencode "preflight=1")
         downstream_preflight=(--preflight)
         shift
         ;;
      --lang)
         lang=$2
         shift 2
         upstream_lang=(--data-urlencode "lang=$lang")
         downstream_lang=(--lang "$lang")
         ;;
      --stop-services)
         stop_services=(--stop-services)
         shift
         ;;
      --stop-cronjobs)
         stop_cronjobs=(--stop-cronjobs)
         shift
         ;;
      --trace)
         trace=(--trace)
         shift
         ;;
      *)
         usage
         exit 1
         ;;
   esac
done
shift $((OPTIND-1))

cmd=(
   /usr/local/pd-admin2/bin/curl
#      -v
      -sS
      -X POST
      "https://${remote_server}/administrator/"
      --data-urlencode todo=export-customer
      --data-urlencode "remote_customer_login=$remote_customer_login"
      --data-urlencode "remote_customer_password=$remote_customer_password"
      --data-urlencode "transfer_protocol_version=1"
      ${upstream_preflight[@]+"${upstream_preflight[@]}"}
      ${upstream_lang[@]+"${upstream_lang[@]}"}
)

cmd2=(
   /opt/pdadmin/bin/import-customer
      --reseller "$reseller"
      ${account[@]+"${account[@]}"}
      ${downstream_preflight[@]+"${downstream_preflight[@]}"}
      ${downstream_lang[@]+"${downstream_lang[@]}"}
      ${stop_services[@]+"${stop_services[@]}"}
      ${stop_cronjobs[@]+"${stop_cronjobs[@]}"}
      ${trace[@]+"${trace[@]}"}
)
nousage=1
"${cmd[@]}" 2>&1 | "${cmd2[@]}"
