#!/bin/bash
#
#  import-pdx.sh
#  BNK/PDA#73
#  2023-04-17 stvo
#
set -CEeuo pipefail
#set -x
nousage=

if [[ ${JOB_ID-} ]]; then
   echo $JOB_ID
   logdir="/tmp/import-customer.$JOB_ID"
   mkdir "$logdir"
   chmod o= "$logdir"
   stdout="$logdir/STDOUT"
   stderr="$logdir/STDERR"
   exec 2<> $stderr
   exec 1<> $stdout
fi

usage ()
{
   echo "(import-pdx.sh:$BASH_LINENO)" >&2
   if (($nousage)); then
      return
   fi
   echo "$0 [options] <customer>.pdx

Options

--reseller                 login (dst)
--account                  account (dst)
--preflight                only validate credentials
--stop-services            disable all imported services
--stop-cronjobs            disable all imported cronjobs
"
}

trap usage ERR

account=()
downstream_preflight=()
downstream_lang=()

while (($#)); do
   case $1 in
      --reseller)
         reseller=$2
         shift 2
         ;;
      --account|--plan)
         plan=$2
         shift 2
         account=(--account "$plan")
         ;;
      --preflight)
         downstream_preflight=(--preflight)
         shift
         ;;
      --lang)
         lang=$2
         shift 2
         downstream_lang=(--lang "$lang")
         ;;
      --stop-services)
         stop_services=(--stop-services)
         shift
         ;;
      --stop-cronjobs)
         stop_cronjobs=(--stop-cronjobs)
         shift
         ;;
      --delete)
         delete=1
         shift
         ;;
      *)
         input=$1
         shift
         break
         ;;
   esac
done
shift $((OPTIND-1))

[[ $# = 0 ]]

cmd2=(
   /opt/pdadmin/bin/import-customer
      --reseller "$reseller"
      ${account[@]+"${account[@]}"}
      ${downstream_preflight[@]+"${downstream_preflight[@]}"}
      ${downstream_lang[@]+"${downstream_lang[@]}"}
      ${stop_services[@]+"${stop_services[@]}"}
      ${stop_cronjobs[@]+"${stop_cronjobs[@]}"}
)
nousage=1
exec 7<"$input"
[[ ${delete-} ]] && rm "$input"
if [[ ${#downstream_preflight[@]} -ne 0 ]]; then
   "${cmd2[@]}" <&7
else
   cat <&7 | "${cmd2[@]}"
fi
