#!/usr/local/pd-admin2/bin/perl -w

#####
#####  PD-Admin 2.0 - quota.pl
#####  (c) 1999 - 2002 by Daniel Bradler
#####

use strict;
use DBI;
use Quota;

#####
#####  database connection 
#####

use vars qw($popuser_name $dsn $user $password $quota_dev 
            $quota_maxfiles $quota_blocks_per_kb $quota_diff_hardlimit);

do "/opt/pdadmin/etc/pdadmin.conf" or die "Can't read configuration: $!";

die "quota_dev is not defined" if ! defined $quota_dev;

my $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
$SIG{__DIE__} = sub {  $dbh->disconnect if $dbh;  die @_;};

#####
#####  defaults
#####

$quota_blocks_per_kb  ||= 1;
$quota_diff_hardlimit ||= 0;

#####
#####  Main
#####

my $query = "select t1.login, t2.discspace from users as t1, accounts as t2 ".
            "where t1.account=t2.id";

my $sth   = $dbh->prepare($query) or die "cannot prepare query '$query'";
my $rv    = $sth->execute or die "cannot execute query '$query'";

while (my ($login, $discspace)  = $sth->fetchrow_array) {
   my ($uid) = (getpwnam($login))[2];
   warn "could not retrieve uid for login <$login>: <$!> <$@>" if ! defined $uid;
   warn "login <$login>: uid = 0: <$!> <$@>" if defined $uid && $uid == 0;
   next if ! $uid;

   my $soft_quota_blocks = $discspace * 1024 * $quota_blocks_per_kb;
   my $hard_quota_blocks = $soft_quota_blocks + $quota_diff_hardlimit;

   my ($bc,$bs,$bh,$bt, $ic,$is,$ih,$it) = Quota::query($quota_dev, $uid, 0);

   $bs ||= 0;
   $bh ||= 0;
   $is ||= $quota_maxfiles || 199999;
   $ih ||= $quota_maxfiles || 199999;

   unless ($bs == $soft_quota_blocks and $bh == $hard_quota_blocks) {
      print "user $login: $bs --> $soft_quota_blocks\n";
      print "user $login: $bh --> $hard_quota_blocks\n";
      Quota::setqlim($quota_dev, $uid, $soft_quota_blocks, $hard_quota_blocks, $is, $ih)
        and warn "Cannot set user quota for $login: $!\n";
   }
}

$sth->finish;
$dbh->disconnect;
