#!/usr/local/pd-admin2/bin/perl -w

use strict;
use DBI;
use Fcntl ':flock';

####
####   Read Configuration
####

use vars qw($popuser_name $dsn $user $password $maillog $syslogd);
           
do "/opt/pdadmin/etc/pdadmin.conf" or die "Can't execute configuration: $!";
my $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
$SIG{__DIE__} = sub {  $dbh->disconnect if $dbh;  die @_;};


####
####  defaults
####

$maillog ||= '/opt/pdadmin/log/LOG.mail-traffic';

####
####  predeclared SUBS
####

sub get_row($) {
  my $query = shift;
  my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'"; 
  my $rv  = $sth->execute or die "cannot execute query '$query'";
  my @row = $sth->fetchrow_array;
  $sth->finish;  
  return @row;
}

sub update_row {
  my $query = shift;
  my $rueck = '';
  my ($sth, $rv, $rc);
  if ($sth = $dbh->prepare($query)) {
    if ($sth->execute) {
      $rueck = $sth->{mysql_insertid} || 1;
    }
    $rc = $sth->finish;
  }
  return $rueck;
}

####
####  Main
####

my $query = "select id, uid, login from users";
my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
my $rv  = $sth->execute or die "cannot execute query '$query'";

my %dbid_uid; my %login_dbid;
while (my ($dbid, $uid, $login) = $sth->fetchrow_array) {
     $dbid_uid{$uid} = $dbid;
     $login_dbid{$dbid} = $login;
}
$sth->finish;

my %domain_dbid;
$query = "select name, owner from domains;";
$sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
$rv  = $sth->execute or die "cannot execute query '$query'";
while (my ($domain, $dbid) =  $sth->fetchrow_array) {

    $domain_dbid{$domain} = $dbid;
}
$sth->finish;

my $time = time;

my $age = int((-M $maillog) * 1440);  # Age of Logfile in min

rename $maillog, "$maillog.$$.$time" or die "cannot rename $maillog: $!";

if (-e "/etc/rsyslog.conf") {
    $syslogd ||= 'rsyslogd';
}

$syslogd ||= 'syslogd';
system "killall -HUP $syslogd";

open LOG, "$maillog.$$.$time" or die "$maillog: $!";

$| = 1;
my %msg; my %delivery; my %traffic;
my %msgs_per_uid; my $msgs_total = 0;
while (<LOG>) {

    chomp;

    if (/info msg (\d+): bytes (\d+) from <([^>]*)> qp \d+ uid (\d+)/) {
        my $msgid = $1;
        my $bytes = $2;
        my $from  = lc $3;
        my $uid   = $4;

        my $owner;

        $msg{$msgid}{bytes} = $bytes;
        $msg{$msgid}{from}  = $from;

        if (defined $dbid_uid{$uid} and $dbid_uid{$uid}) {
            $msg{$msgid}{sender} = $dbid_uid{$uid};
            #print "$msgid bytes $bytes ---> $msg{$msgid}{sender}\n";
        } else {
            if (defined $from and $from =~ /\@/) {
                my $domain = (split /\@/, $from)[-1];
                if (defined $domain_dbid{$domain} and $domain_dbid{$domain}) {
                   $msg{$msgid}{sender} = $domain_dbid{$domain};
                   #print "$msgid bytes $bytes ---> $msg{$msgid}{sender}\n";
                }
            }
        } 

    }

    if (/end msg (\d+)/) {
       my $msgid = $1;

       for my $deliveryid (@{$msg{$msgid}{deliveries}}) {
          if ($delivery{$deliveryid}{msgid} != $msgid) {
              print "msg id $msgid <-> $deliveryid (msgid $delivery{$deliveryid}{msgid})\n";
          }
          unless (defined $msg{$msgid}{bytes}) {
              $msg{$msgid}{bytes}  = 0;
              #print "f: $msgid\n";
          }
          if (defined $delivery{$deliveryid}{status} and $delivery{$deliveryid}{status}) {
              # Zustellung erfolgreich
              if (defined $msg{$msgid}{sender} and $msg{$msgid}{sender}) {
                  # Lokaler Absender -> Anzahl Nachrichten + Traffic aufsummieren
                  $msgs_total++;
                  $msgs_per_uid{$msg{$msgid}{sender}} ||= 0;
                  $msgs_per_uid{$msg{$msgid}{sender}}++;
                  $traffic{$msg{$msgid}{sender}} ||= 0;
                  $traffic{$msg{$msgid}{sender}}  += $msg{$msgid}{bytes};
              }
              if (defined $delivery{$deliveryid}{rcpt} and $delivery{$deliveryid}{rcpt}) {
                  # Lokaler Empfaenger -> Traffic aufsummieren
                  $traffic{$delivery{$deliveryid}{rcpt}} ||= 0;
                  $traffic{$delivery{$deliveryid}{rcpt}}  += $msg{$msgid}{bytes};
              }
          }
          delete $delivery{$deliveryid};
       }
       delete $msg{$msgid};
    }   


    if (/starting delivery (\d+): msg (\d+).*\@([^ ]+)/) {
        my $deliveryid = $1;
        my $msgid = $2;
        my $rcpt  = lc $3;
        my $domain = (split /\@/, $rcpt)[-1];
        push @{$msg{$msgid}{deliveries}}, $deliveryid;
        $delivery{$deliveryid}{msgid} = $msgid;
        if (defined $domain_dbid{$domain} and $domain_dbid{$domain}) {
             $delivery{$deliveryid}{rcpt}  = $domain_dbid{$domain};
        }
    }

    if (/delivery (\d+): success/) {
        $delivery{$1}{status} = 1;
    }
         
}
my $traffic;
for my $dbid (keys %traffic) {
   $msgs_per_uid{$dbid} ||= 0;
   if ($msgs_per_uid{$dbid} > 100) {
      print "$login_dbid{$dbid} --> $traffic{$dbid} bytes // $msgs_per_uid{$dbid} deliveries\n";
   }
   $traffic += $traffic{$dbid};
   my ($test) = get_row "select id from traffic_new where id_user='$dbid' and date=now();";
   if (defined $test and $test) {
     $query = "update traffic_new set mail=mail+$traffic{$dbid} where id_user='$dbid' and date=now();";
  } else {
    $query = "insert into traffic_new (id_user, mail, date) values ('$dbid','$traffic{$dbid}', now())
;";
  }
  unless (update_row $query) {
    warn "Kann Traffic fuer $_ nicht in die Datenbank schreiben!\n$query\n\n";
  }

}

if ($msgs_total > 250) {
   print "\n\ntotal msgs sent in $age min: $msgs_total\n"
}

unlink "$maillog.$$.$time";

#for my $msgid (keys %msg){
#   $msg{$msgid}{bytes} ||= 0;
#   $msg{$msgid}{from}  ||= ''; 
#   print join(';', $msgid, $msg{$msgid}{bytes}, 
#              $msg{$msgid}{from}, @{$msg{$msgid}{deliveries}}), 
#         "\n";
#}
#for my $deliveryid (keys %delivery) {
#   $delivery{$deliveryid}{msgid} ||= 0;
#   $delivery{$deliveryid}{rcpt} ||= '';
#   print join(';', $delivery{$deliveryid}{msgid}, $delivery{$deliveryid}{rcpt}), "\n"
#}
