#!/usr/local/pd-admin2/bin/perl -w

use strict;

use LWP::UserAgent;
use HTTP::Request;

########################
sub comodo_hashes($) {
########################

   my $csr = shift;

   my $browser = LWP::UserAgent->new;
   my $response = $browser->post(
        'https://secure.comodo.net/products/!DecodeCSR',
        [ csr            => $csr,
          showCSRHashes  => 'Y',
          showFieldNames => 'N',
          showCN         => 'N',
          showAddress    => 'N',
        ]
      );

   unless ($response->is_success) {
     warn $response->status_line;
     return;
   }

   my $result = $response->content;
   my ($md5) =  $result =~ m,md5=(.*),;
   my ($sha1) = $result =~ m,sha1=(.*),;

   return $md5, $sha1;

}
    
############################
sub bnk_order_cert($$$$$$) {
############################

   my $hostname    = shift;
   my $lifespan    = shift;
   my $csr         = shift;
   my $ssl_user    = shift;
   my $ssl_pass    = shift;
   my $feedbackurl = shift;

   ($lifespan) = $lifespan =~ m/^(\d+)$/;
   $lifespan ||= 1;

   unless (defined $ssl_user and defined $ssl_pass) {
      exit 60;
   }

   my $browser = LWP::UserAgent->new;
   my $response = $browser->post(
        'https://sslbot.virtualhosts.de/gateway.api',
        [ command        => 'order_cert',
          csr            => $csr,
          user           => $ssl_user,
          password       => $ssl_pass,
          commonname     => $hostname,
          lifespan       => $lifespan,
          feedbackurl    => $feedbackurl,
          product        => 'Single',
        ]
   );

   my $result = $response->content;

   if ($result =~ /SUCCESS::OrderID (\d+)/) {
      print $1;
      exit;
   } else {
      $result =~ /ERROR::([^:]+)/;
      print $1;
      exit 66;
   }

}

###
### main()
###

if (defined $ENV{action} and $ENV{action} eq 'comodo_hashes') {
    if (defined $ENV{csr} and length $ENV{csr}) {
       my ($md5, $sha1) = comodo_hashes $ENV{csr};
       if (defined $md5 and defined $sha1) {
          print $md5, "\n", $sha1, "\n";
          exit;
       } else {
          exit 70;
       }
    }
}

if (defined $ENV{action} and $ENV{action} eq 'order_cert') {
    bnk_order_cert $ENV{hostname}, $ENV{lifespan}, $ENV{csr}, $ENV{ssl_user}, $ENV{ssl_pass}, $ENV{url};
}

exit 2;
