#!/usr/local/pd-admin2/bin/perl -w

use strict;

####
####  pd-Admin 2.0 - statistik.pl
####

use strict;
use DBI;

####
####  configuration
####

use vars qw($dsn $user $password $usage_dir $sth $webalizer_path $access_log_rotation);

do "/opt/pdadmin/etc/pdadmin.conf" or die "Can't execute configuration: $!";
my $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
$dbh->{InactiveDestroy} = 1;
$SIG{__DIE__} = sub {  $dbh->disconnect if $dbh;  die @_;};

$webalizer_path      ||= '/usr/local/bin/webalizer';
$access_log_rotation ||= '50MB';

my $mday = (localtime(time))[3]; 
my $wday = (localtime(time))[6]; 

####
####   S U B s
####

sub get_login($) {
  my $id = shift;
  my $query = "select login from users where id='$id'";
  my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
  my $rv  = $sth->execute or die "cannot exucte query '$query'";
  my ($loginid) = $sth->fetchrow_array;
  $sth->finish;
  return $loginid;
}

sub getdbuid($) {
  my $loginid = shift;
  my $query = "select id from users where login='$loginid'";
  my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
  my $rv  = $sth->execute or die "cannot exucte query '$query'";
  my ($id) = $sth->fetchrow_array;
  $sth->finish;
  return $id;
}

sub webalizer($$$$$$$) {

  my $user     = shift;
  my $conf     = shift;
  my $hostname = shift;
  my $logpath  = shift;
  my $htdocs   = shift;
  my $home     = shift;
  my $stats    = shift;

  unless (-d $htdocs) {
    mkdir $htdocs,0755 or warn "$htdocs: $!";
  }

  my $pid = fork;
  if (defined $pid) {
    unless ($pid) {
       $SIG{__DIE__} = sub { die @_; };
       $sth->finish;
       my ($uid, $gid) = (getpwnam($user))[2,3];
       chown $uid, $gid, $htdocs;
       $( = $gid; $) = $gid; $< = $uid; $> = $uid;
       unless ($stats eq 'off') {
          system "mkdir -p $htdocs";
          if (-e "$home/webalizer.conf" and not -l "$home/webalizer.conf") {
              open CONF, "$home/webalizer.conf" or die "$home/webalizer.conf: $!";
              $conf = join "", <CONF>;
              close CONF;
          }
          $conf =~ s/\$\$logfile_path/$logpath/g;
          $conf =~ s/\$\$stats_path/$htdocs/g;
          $conf =~ s/\$\$hostname/$hostname/g;
          open CONF,">$htdocs/.webalizer.conf" or die "$htdocs/.webalizer.conf: $!: $!\n";
          print CONF $conf or die "Cannot write $htdocs/.webalizer.conf: $!\n";
          close CONF or die "Cannot close $htdocs/.webalizer.conf nicely: $!\n";
          my $ausgabe = `$webalizer_path -c $htdocs/.webalizer.conf`;
       }

       my $split_log = 0;

       $split_log = 1 if ($access_log_rotation eq '50MB'    and -s $logpath > 5000000);
       $split_log = 1 if ($access_log_rotation eq 'weekly'  and $wday == 0);
       $split_log = 1 if ($access_log_rotation eq 'monthly' and $mday == 1);
       $split_log = 1 if ($access_log_rotation eq 'daily');

       if ($split_log) {
          my $date  = sprintf "%04d%02d%02d", 1900 + (localtime(time))[5],
                                              (localtime(time))[4]+1,
                                              (localtime(time))[3];
          unless (-d "$home/logs/archive") {
             mkdir "$home/logs/archive", 0750 or warn "cannot mkdir $home/logs/archive: $!\n";
          }
          rename $logpath, "$home/logs/archive/$hostname-access.log.$date" or warn "cannot rename $logpath: $!";
          system 'gzip', "$home/logs/archive/$hostname-access.log.$date";
       }
       exit;
    } else {
       waitpid $pid,0
    }
  } else {
    warn "cannot fork: $!\n" 
  }

}

####
####  query
####

$usage_dir ||= 'usage2';

if (-e "/opt/pdadmin/logs/webalizer.lastlog") {
  system ("rm /opt/pdadmin/logs/webalizer.lastlog");
}

open CONF, "/opt/pdadmin/etc/webalizer.conf.master" or die "webalizer.conf.master: $!";
my $masterconf = join "", <CONF>;
close CONF;

my $query = "select t1.name, t1.vhostroot, t2.name, t1.owner, t3.webalizer, t2.target, ". 
            "t2.webalizer_dir from domains as t1, vhosts as t2, users as t3 where t2.domain=t1.id and t1.owner=t3.id;";
   $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
my $rv  = $sth->execute or die "cannot execute query '$query'";

while (my @row = $sth->fetchrow_array) {
   my $domain;
   my $owner = $row[3];

   if (defined $row[2] and length $row[2]) {
      $domain = "$row[2].$row[0]";
   } else {
      $domain = $row[0];
   }
   print "Bearbeite $domain.\n"; 
   my $user  = get_login $owner;
   next unless (defined $user and defined $user);
   
   my $home  = (getpwnam $user)[7];
   next unless (defined $home and defined $home);

   my $log    = "$home/logs/$domain-access.log";

   my $htdocs;
   my $target = $row[5]; 
   if (defined $row[6] and length $row[6]) {
      $htdocs = "$row[1]/$row[6]"
   } else {
      if (defined $target and length $target and $target !~ m!^https?://!) {
         if ($target ne "$row[1]/$domain") {
            $htdocs = "$target/$usage_dir/$domain";
         } else {
            $htdocs = "$row[1]/$domain/$usage_dir";
         }
      } else { 
         $htdocs = "$row[1]/$domain/$usage_dir";
      }
   }
   if (-e $log) {
     webalizer $user, $masterconf, $domain, $log, $htdocs, $home, $row[4];
   }
}
$sth->finish;
