#!/usr/local/pd-admin2/bin/perl -w

####
####   pd-admin 2.x - traffic-warning.pl 
####   Trafficauswertung (c) 2001-2003 by Daniel Bradler
####

use strict;
use DBI;

#####
#####   V A R
#####

use vars qw($driver $database $table $hostname $dsn $user $password
            $warn_interval);

#####
#####   Database connection
#####

do "/opt/pdadmin/etc/pdadmin.conf" or die "Can't do execute configuration: $!";
my $dbh = DBI->connect($dsn, $user, $password) or die "can't connect!";
$SIG{__DIE__} = sub {  $dbh->disconnect if $dbh;  die @_;};

####
####   vars 
####

my %email;
my %toomuch;

$warn_interval = $warn_interval || 7 * 86400; # default 7 Tage
my $this_month = sprintf "%04d-%02d", 1900 + (localtime(time))[5],
                                       (localtime(time))[4]+1;

####
####   predeclared subs
####

###############################
sub check_home($) {
###############################
   my $home = shift;
   ($home)  = $home =~ m#^(/[\w/.-]+)$#;
   return $home;
}

sub get_home($) {
   my $user = shift;
   my $home = (getpwnam($user))[7];
   $home    = check_home($home);
   return $home;
}

sub update_row {
  my $query = shift;
  my $rueck = '';
  my ($sth, $rv, $rc);
  if ($sth = $dbh->prepare($query)) {
    if ($sth->execute) {
      $rueck = $sth->{mysql_insertid} || 1;
    }
    $rc = $sth->finish;
  }
  return $rueck;
}

###############################
sub human_readable($) {
###############################
  my $traffic = shift;
  return sprintf "%.3f GB", $traffic / 1000000000;
}

###############################
sub get_row($@) {
###############################
  my $query = shift;
  my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
  my $rv  = $sth->execute(@_) or die "cannot execute query '$query'";
  my @row = $sth->fetchrow_array;
  $sth->finish;
  return @row;
}

###############################
sub getdbuid($) {
###############################
  my $loginid = shift;
  my $query = "select id from users where login='$loginid'";
  my $sth = $dbh->prepare($query) or die "cannot prepare query '$query'";
  my $rv  = $sth->execute or die "cannot execute query '$query'";
  my ($id) = $sth->fetchrow_array;
  $sth->finish;
  return $id;
}

###############################
sub user_traffic($) {
###############################
   
   my $user  = shift;
   my $dbuid = getdbuid $user;

   return 0 unless (defined $dbuid and $dbuid);

   my $query = "select sum(www), sum(ftp), sum(mail) from traffic_new ".
               "where date like '$this_month%' and id_user='$dbuid'";

   my ($traffic_www,
       $traffic_ftp,
       $traffic_mail) = get_row $query;

   $traffic_www ||= 0; $traffic_ftp ||= 0; $traffic_mail ||= 0;

   my $traffic        = $traffic_www + $traffic_ftp + $traffic_mail;
   my ($limit)        = get_row "select t1.traffic from accounts as t1, users as t2 ".
                                "where t1.id=t2.account and t2.id=$dbuid;";
   $limit             = $limit || 0;
   $limit            *=  1000  * 1000;

   return ($traffic, $limit);
}

###############################
sub get_email($) {
###############################

   my $email;
   my $reseller = shift;
   my $home     = get_home $reseller;
   my ($remail) = get_row 'select email from resellers where login=?', $reseller;
   my $from;
   if (defined $remail and $remail) {
      $from = "Traffic-Auswertung <$remail>";
   } else {
      $from = 'Traffic-Auswertung: ;';
   }

   if (defined $home and length $home) { 
      if (open TPL, "$home/reseller/email.traffic") {
         $email = join '', <TPL>;
         close TPL;
         $email =~ y/\r//d;
      }
   }

   unless (defined $email and length $email) {
      ($email = <<_) =~ s/^[ \t]+//gm;
         From: $from
         To: \$\$email
         Subject: Traffic-Ueberschreitung

         Sehr geehrter Benutzer,

         leider haben Sie Ihr Transfervolumen fuer diesen Monat 
         ueberschritten. 

         Account:     \$\$user
         Freivolumen: \$\$limit
         Verbrauch:   \$\$traffic
         Differenz:   \$\$diff 
        
         Mit freundlichen Gruessen,
         Ihr Internet Service Provider 
_
    }

    return $email;

}

###############################
sub send_email($) {
###############################
   
   my $email = shift;
   if (open MAIL, "| /var/qmail/bin/qmail-inject") {
      print MAIL $email;
      if (close MAIL) {   
         return 1;
      } else {
         warn "cannot close qmail-inject nicely: $!";
         return 0;
      }      
   } else {
     warn "cannot open qmail-inject: $!";
     return 0;
   }

}

###############################
sub warning($$$$) {
###############################

  ####
  ####  ggf. Warnung verschicken
  ####

  my $reseller      = shift;
  my $user          = shift;
  my $traffic       = shift; 
  my $limit         = shift;
  my $diff          = human_readable($traffic - $limit);
     $traffic       = human_readable($traffic);
     $limit         = human_readable($limit);

  $toomuch{$reseller}{$user} = $diff; 

  my ($last_warned, 
      $email)       = get_row "select traffic_warning, email from users where login='$user'";  

  if (time - $last_warned >= $warn_interval) {

      ### Warnung verschicken

      print "Verschicke Warnung an $user ...\n";

      unless (defined $email{$reseller} and length $email{$reseller}) {
         $email{$reseller} = get_email $reseller
      }

      my $text = $email{$reseller};

      $text =~ s/\$\$email/$email/g;
      $text =~ s/\$\$limit/$limit/g;
      $text =~ s/\$\$diff/$diff/g;
      $text =~ s/\$\$traffic/$traffic/g;
      $text =~ s/\$\$user/$user/g;

      if (send_email $text) {
         update_row "update users set traffic_warning='@{[time]}' where login='$user'";
      }

  }

}

#### 
####   M A I N
####

my $query = 'select t1.login, t2.login from users as t1, resellers as t2 '.
            "where t1.reseller=t2.id";
my $sth   = $dbh->prepare($query) or die "cannot prepare query '$query'";
my $rv    = $sth->execute or die "cannot execute query '$query'";

my %traffic;
while (my ($user, $reseller) = $sth->fetchrow_array) {
   my ($traffic, $limit) = user_traffic $user;
   warning($reseller, $user, $traffic, $limit) if ($traffic > $limit);
   $traffic{$reseller} += $traffic;
}
$sth->finish;
