#!/usr/local/pd-admin2/bin/perl -w

#####
#####   pd-admin 4 - spamfilter.pl
#####   (c) 2001 - 2007 by Daniel Bradler
#####

use strict;
use Unix::Syslog qw(:macros);
use Unix::Syslog qw(:subs);

####
####  Syslogging
####

openlog('spamfilter', LOG_PID, LOG_MAIL);

####
####   which user?
####

my ($user) = (split /-/, $ENV{LOCAL})[0];
my $exitcode;


if (open EXIT, "/home/popuser/spamfilter_exitcode") {
   $exitcode = <EXIT>;
   chomp $exitcode;
   close EXIT;
}

$exitcode ||= 99;

# Spamfilter enabled?
my $filter = 0; my $quarantine_maildir;

if (open SPAMFILTER, "/home/popuser/spamfilter") {
   while (defined ($_ = <SPAMFILTER>)) {
      chomp;
      my ($test, $q) = split /:/, $_, 2;
      if ($user eq $test) {
         $filter = 1;
         $quarantine_maildir = $q;
         last;
      }
   }
   close SPAMFILTER;
}

if (defined $ENV{FORCEFILTER} and $ENV{FORCEFILTER}) {
    $filter = 1;
}

exit 0 unless $filter;

#print "Spamfilter invoked for user='$user', quarantine_maildir='$quarantine_maildir'\n";

# Backscatter filter enabled?
my $filter_backscatter = 0;
if (open SPAMFILTER, "/home/popuser/spamfilter_backscatter") {
   while (defined ($_ = <SPAMFILTER>)) {
      chomp;
      if ($user eq $_) {
         #$filter_backscatter = 1;
         $filter_backscatter = 0; # temporarily disabled
         last;
      }
   }
   close SPAMFILTER;
}

# envelope sender?
my $sender = lc $ENV{SENDER};
my $recipient = lc $ENV{RECIPIENT};

# Read email header
my $header = ''; my $leerzeile = 0;
while (<>) {
   if (/^$/) {
      if ($leerzeile) {
         last
      } else {
         $leerzeile = 1;
      }
   }
   $header .= $_;
}

# read body (first 150 lines)
my $body = ''; my $counter = 0;
while (<>) {
   $counter++;
   $body .= $_;
   last if ($counter >= 150);
}

####
####   prepare for spam checking
####

my $spam = '';

# user's whitelist
if (open WHITELIST, "/home/popuser/whitelists/$user.txt") {
   while (defined ($_ = <WHITELIST>)) {
      chomp;
      next unless /\@/;
      if (/^\@[a-z0-9.-]+$/) {
         if (lc $sender =~ /$_$/) {
            exit;
         }
      }
      if (lc $sender eq lc $_) {
          exit;
      }
   }
   close WHITELIST;
}

# user's blacklist 
unless (length $spam) {
   if (open BLACKLIST, "/home/popuser/blacklists/$user.txt") {
      while (defined ($_ = <BLACKLIST>)) {
         chomp;
         next unless /\@/;
         if (/^\@[a-z0-9.-]+$/) {
            if (lc $sender =~ /$_$/) {
               $spam = "envelope sender is listed in my blacklist.";
               last
            }
         }
         if (lc $sender eq lc $_) {
             $spam = "envelope sender is listed in my blacklist.";
         }
      }
      close BLACKLIST;
  }
}

# check for SpamAssassin's X-Spam-Flag
unless (length $spam) {
   if ($header =~ /^X-Spam-Flag:\s+YES/m or $header =~ /^X-Spam-Status:\s+YES/mi) {
      $spam = 'Message tagged as spam by SpamAssassin.';
   }
}

unless (length $spam) {
   if ($filter_backscatter) {
      my $bounce = 0;

      if ($header =~ /^From:\s+mailer-daemon\@/mi and $header =~ /^Subject:\s+failure notice/mi) {
         $bounce = 1;
      }
      if ($header =~ /^From:\s+postmaster\@/mi and $header =~ /^Subject:\s+failure notice/mi) {
         $bounce = 1;
      }

      if ($header =~ /^From:.*mailer-daemon\@/mi and $header =~ /^Subject:\s+Undelivered Mail Returned to Sender/mi) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*mailer-daemon\@/mi and $header =~ /^Subject:\s+Returned mail/mi) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*mailer-daemon\@/mi and $header =~ /^Subject:\s+Mail delivery failed/mi) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*postmaster\@/mi and $header =~ /^Subject:\s+Delivery Status Notification/mi) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*postmaster\@/mi and $header =~ /^Subject:\s+Mail Delivery Subsystem/mi) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*mailer-daemon\@/mi and $header =~ /^Subject:\s+Undeliverable mail/) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*postmaster\@/mi and $header =~ /^Subject:\s+Undeliverable mail/) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*mailer-daemon\@/mi and $header =~ /^Subject:\s+Mail Delivery Failure/) {
         $bounce = 1;
      }
      if ($header =~ /^Subject:\s+Your e-mail could not be delivered/) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*postmaster\@/mi and $header =~ /^Subject:\s+Message you sent blocked by our bulk email filter/) {
         $bounce = 1;
      }
      if ($header =~ /^From:.*Barracuda Spam Firewall/mi) {
         $bounce = 1;
      }




      if ($bounce) {
#          if (open ME, "/var/qmail/control/me") {
#             my $pda_domain = <ME>;
#             close ME;
#             chomp $pda_domain;
#             ($pda_domain) = $pda_domain =~ /[a-z0-9-]\.([a-z0-9.-]+)$/;
             my $body =~ /X-PDA-ORIGIN: [a-z0-9-]\.([a-z0-9.-]+)$/;
             my $pda_origin_domain = $1;
             unless (defined $pda_origin_domain and length $pda_origin_domain) {
                 $spam = 'Probably Backscatter.'
             }
#          } else {
#             syslog LOG_INFO, "Cannot open /var/qmail/control/me: $!";
#          }
     }
   }
}

# exit if not spam
exit 0 unless ($spam);

# syslogging 
syslog LOG_INFO, "$sender->$recipient: $spam";

# quarantine spam
if (defined $quarantine_maildir and $quarantine_maildir and -d $quarantine_maildir) {
   if (open SAFECAT, "|/home/popuser/safecat $quarantine_maildir/tmp $quarantine_maildir/new") {
      print SAFECAT $header;
      print SAFECAT $body;
      print SAFECAT "\n";
      while (<>) {
         print SAFECAT $_
      }
      unless (close SAFECAT) {
         print "cannot close safecat nicely\n";
         exit 111
      } else {
         syslog LOG_INFO, "$sender->$recipient: Spam mail saved to quarantine maildir <$quarantine_maildir>";
         exit 99;
      }
   } else {
      print "failed to safe mail to quarantine maildir\n";
      exit 111
   }
} else {
   print $spam unless ($exitcode == 99);
   exit $exitcode
}

exit 0;
