#!/bin/bash
#
# 2002-02-25
# 2004-09-08 stvo: fastforward
# 2004-12-30 stvo: prefetch files
# 2005-03-29 stvo: simscam 
#
# $Id: qmail2.sh,v 1.44 2021/07/05 13:43:23 testx Exp testx $
#

#######################################################
#######  Installation der Maildienste fuer pd-admin
#######  (c) 1999 bis 2003  Daniel Bradler
#######################################################


if [ -d /usr/local/pd-admin2/qmail/owners ]; then
   HASQMAIL=true
else
   HASQMAIL=false
fi

#
#  prefetch source directory
#
PFS=/usr/src

if [ "$SERVERNAME" == "" ]; then
	echo "SERVERNAME ist nicht gesetzt."
	exit 47
fi

echo "Installiere Maildienste fuer pd-admin."
echo

installdir=`pwd`
runlevel=`runlevel | cut -d' ' -f2`

chmod 751 /etc
chmod 751 /home

PATH=/usr/local/pd-admin2/bin:$PATH

######################################
####  qmail & Zubehoer installieren
######################################

###########################################
####  User- und Gruppen-Accounts anlegen
##########################################

## /var/qmail anlegen
mkdir -p /var/qmail

## Gruppen fuer qmail anlegen

if ! $HASQMAIL; then

grep "^nofiles:" /etc/group >/dev/null 2>&1 || {
  groupadd nofiles
}

grep "^qmail:" /etc/group >/dev/null 2>&1 || {
  groupadd qmail
}

## User fuer qmail anlegen
grep "^alias:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g nofiles -d /var/qmail/alias alias
}

grep "^qmaild:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g nofiles -d /var/qmail qmaild
}

grep "^qmaill:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g nofiles -d /var/qmail qmaill
}

grep "^qmailp:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g nofiles -d /var/qmail qmailp
}

grep "^qmailq:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g qmail -d /var/qmail qmailq
}

grep "^qmailr:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g qmail -d /var/qmail qmailr
}

grep "^qmails:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g qmail -d /var/qmail qmails
}

fi

#  Home fuer spamd anlegen
mkdir -p /var/spamd

## Gruppe fuer SA anlegen
grep "^spamd:" /etc/group >/dev/null 2>&1 || {
  groupadd spamd
}

## User fuer SA anlegen
grep "^spamd:" /etc/passwd >/dev/null 2>&1 || {
  useradd -g spamd -d /var/spamd -s /bin/false spamd
}

chown spamd.spamd /var/spamd

#sc#### Gruppe fuer qmail-scanner anlegen
#sc#grep "^qscand:" /etc/group >/dev/null 2>&1 || {
#sc#  groupadd qscand
#sc#}
#sc#
#sc### User fuer qmail-scanner anlegen
#sc#grep "^qscand:" /etc/passwd >/dev/null 2>&1 || {
#sc#  useradd -c "Qmail-Scanner Account" -g qscand  -s /bin/false qscand
#sc#}

### Gruppe fuer clamav anlegen
#grep "^clamav:" /etc/group >/dev/null 2>&1 || {
#  groupadd clamav
#}

## User fuer clamav anlegen
#grep "^clamav:" /etc/passwd >/dev/null 2>&1 || {
#  useradd -g clamav -s /bin/false -c "Clam AntiVirus" clamav
#}

########################################
#### qmail runterladen und entpacken
########################################

cd /usr/src

if ! $HASQMAIL; then

test -e qmail-1.03 && mv qmail-1.03 qmail-1.03.`date +%`

tar -xzf $PFS/qmail-1.03.tar.gz || {
   echo "kann qmail nicht entpacken"
   exit 1
}
fi

###########################
###  Install daemontools
###########################


rm -f /usr/local/bin/tcpserver

mkdir -p /packages
cd /packages
tar -xzf $PFS/daemontools-0.76.tar.gz
cd /packages/admin/daemontools-0.76/

patch -p 1 < $PFS/daemontools-0.76.errno.patch || {
    echo "kann daemontools-0.76.errno.patch nicht anwenden"
    exit 1
}

kill -0 1 2>/dev/null || {
   echo "daemontools-vhost-Patch wird eingespielt"
   patch -p2 -i $HERE/patches/daemontools-vhost.patch
   echo "success = $?"
   sleep 2
}

./package/install || {
   echo "kann daemontools-0.76 nicht installieren"
   exit 1
} 

kill -0 1 2>/dev/null || {
   echo 
   echo "WARNUNG:  Dieses System ist kein Standard-System.  Es ist"
   echo "nicht sichergestellt, dass der svscanboot-Prozess beim naechsten"
   echo "Reboot des Server gestartet wird.  Ggf. muessen Sie den Prozess"
   echo "manuell mit dem Befehl '/command/svscanboot &' starten."
   echo 
}

#
#  in case of upstart (old)
#
if [ -d /etc/event.d/ ]; then
cat <<EOT >/etc/event.d/svscanboot
start on runlevel [123456]
respawn
exec /command/svscanboot
EOT
initctl start daemontools
fi

#
#  upstart (newer)
#
if test -d /etc/init; then

UPSTART=1

cat <<__ >/etc/init/daemontools.conf
start on runlevel [123456]
respawn
exec /command/svscanboot
__
initctl start daemontools
fi

#
#  if systemd is used
#
test -e /lib/systemd/system/ && {

SYSTEMD=1

cat <<__ > /lib/systemd/system/daemontools.service
[Unit]
Description=DJB daemontools
After=sysinit.target

[Service]
ExecStart=/command/svscanboot
Restart=always

[Install]
WantedBy=multi-user.target
__

ln -sf /lib/systemd/system/daemontools.service /etc/systemd/system/multi-user.target.wants/
systemctl daemon-reload
systemctl start  daemontools
}

#
#  if none of init, upstart, systemd is available
#
if [ "$SYSTEMD" == "" ] &&  [ "$UPSTART" == "" ] ; then

killall -HUP init || {
   /command/svscanboot &
   cat <<ENDE > /usr/local/pd-admin2/sbin/daemontools.rc
#!/bin/bash
case "\$1" in
    start)
        /command/svscanboot &
        ;;
    stop)
        killall svscanboot
        ;;
    *)
        echo "Usage: $0 {start|stop}"
        exit 1
esac
exit 0
ENDE
   chmod 750 /usr/local/pd-admin2/sbin/daemontools.rc
   ln -fs /usr/local/pd-admin2/sbin/daemontools.rc \
      /etc/rc$runlevel.d/S20daemontools
}

fi

killall -0 svscanboot 2>/dev/null || {
   echo 
   echo "Starte svscanboot manuell."
   echo 
   setsid /command/svscanboot &
}


######################
### Install ucspi-tcp
######################

cd /usr/src
rm -fr ucspi-tcp-0.88
tar -xzf $PFS/ucspi-tcp-0.88.tar.gz
cd ucspi-tcp-0.88

patch -p1 -i $PFS/ucspi-tcp-0.88-ipv6.diff19 || {
    echo "kann ucspi-tcp-0.88-ipv6.diff19 nicht anwenden"   
    exit 1
}

patch -p1 -i $PFS/tcpserver-limits-2006-01-26-ipv6.diff || {
    echo "kann tcpserver-limits-2006-01-26-ipv6.diff nicht anwenden"
    exit 1
}

make && make setup check || {
    echo "kann ucspi-tcp-0.88.tar.gz nicht kompilieren"
    exit 1
}

######################
###  Install tcprules
######################

#sc#echo ':allow,QMAILQUEUE="/var/qmail/bin/qmail-scanner-queue.pl"' > /etc/tcp.smtp
#sc#tcprules /etc/tcp.smtp.cdb /etc/tcp.smtp.tmp < /etc/tcp.smtp

###############################
###  glibc 2.3 patch
###############################

if ! $HASQMAIL; then

rm -f /var/qmail/bin/qmail-lspawn
rm -f /var/qmail/bin/qmail-rspawn
rm -f /var/qmail/bin/qmail-clean
rm -f /var/qmail/bin/qmail-send
rm -f /var/qmail/bin/splogger

cd /usr/src/qmail-1.03
patch < $PFS/qmail-1.03.errno.patch || {
    echo "Kann glibc 2.3 Patch nicht anwenden"
    exit 1
}
cd /usr/src/qmail-1.03
patch -p1 -i $PFS/qmail-1.03.dns.patch || {
    echo "Kann qmail-DNS-patch nicht anwenden"
    exit 1
}

fi

###############################
###  QMAILQUEUE Patch anwenden
###############################

if ! $HASQMAIL; then

cd /usr/src/qmail-1.03
patch < $PFS/qmail-queue.patch  || {
  echo "Kann qmail-queue.patch nicht anwenden"
  exit 1
}

fi

##############################
###  SMTP-Auth Patch anwenden
##############################

if ! $HASQMAIL; then
cd /usr/src
tar -xzf $PFS/qmail-smtpd-auth-0.31.tar.gz
cd qmail-smtpd-auth-0.31
cp README.auth base64.c base64.h ../qmail-1.03
patch -d ../qmail-1.03 < auth.patch || {
   echo "kann qmail-smtpd-auth-Patch nicht anwenden"
   exit 1
}
fi

#################################
###  Auth CRAM-MD5 deaktivieren
#################################

if ! $HASQMAIL; then

cd /usr/src/qmail-1.03
cat <<ENDE > no-cram-md5.patch
--- qmail-smtpd.c-orig  Sat Sep 27 15:31:02 2003
+++ qmail-smtpd.c       Sat Sep 27 15:35:29 2003
@@ -26,7 +26,7 @@
 #include "wait.h"
 #include "fd.h"

-#define AUTHCRAM
+// #define AUTHCRAM
 #define MAXHOPS 100
 unsigned int databytes = 0;
 int timeout = 1200;
ENDE
patch < no-cram-md5.patch || {
   echo "Kann no-cram-md5.patch nicht anwenden"
   exit 1
}

fi

###############################
### qmail-queue custom erors
###############################

if ! $HASQMAIL; then

cd /usr/src/qmail-1.03
patch -p1 -i $HERE/patches/N2.patch  || {
  echo "Kann N2.patch nicht anwenden"
  exit 1
}

#
#  2006-06-28 stvo
#  spp patch
#
cd /usr/src
tar xfz qmail-spp-0.42.tar.gz
cd /usr/src/qmail-1.03
patch -p0 -i ../qmail-spp-0.42/qmail-spp-smtpauth.diff || {
  echo "Kann qmail-spp patch nicht anwenden"
  exit 1
}

fi

#
#  plugin anlegen
#
mkdir -p /var/qmail/plugins
cat <<EOT > /var/qmail/plugins/chkauth.sh
#!/bin/bash

if [ "\$SMTPAUTHUSER" == "" ]
   then
      echo "R550 sorry, SMTP-AUTH is required to use the MSA port (#5.7.0 - chkauth)"
      echo "chkauth : Mail from \$TCPREMOTEIP rejected, no SMTP-AUTH" >&2
   fi
EOT
chmod +x /var/qmail/plugins/chkauth.sh
mkdir -p /var/qmail/control
echo 12 >/var/qmail/control/concurrencyincoming
cat <<EOT > /var/qmail/control/msaplugins
[connection]

[auth]

[helo]

[mail]
plugins/chkauth.sh

[rcpt]

[data]
EOT

#
#  2006-06-22 stvo
#  checkuser patch (vadmin-patch)
#

if ! $HASQMAIL; then

cd /usr/src/qmail-1.03
patch -p1 -i ../qmail-checkuser3.patch || {
  echo "Kann qmail-checkuser.patch nicht anwenden"
  exit 1
}
patch -p1 -i ../qmail-xot.patch || {
  echo "Kann qmail-xot.patch nicht anwenden"
  exit 1
}

fi

#######################
#### 32-bit linking in 64-bit environment
#######################

if ! $HASQMAIL; then 

gcc -v 2>&1 | grep -q 2.95  || {
   DEBIAN=`cat /etc/debian_version 2>/dev/null`
   if [ "$DEBIAN" == "3.1" ]; then
      echo 'gcc-3.4 -O2 -m32' > conf-cc
      echo 'gcc-3.4 -m32 -s' > conf-ld
   else
      echo 'gcc -O2 -m32' > conf-cc
      echo 'gcc -m32 -s' > conf-ld
   fi
}

fi


#######################
#### qmail kompilieren
#######################

if $HASQMAIL; then

   ln -s /usr/local/pd-admin2/qmail/bin /var/qmail
   ln -s /usr/local/pd-admin2/qmail/boot /var/qmail
   ln -s /usr/local/pd-admin2/qmail/owners /var/qmail
   /usr/local/pd-admin2/qmail/bin/queue-fix /var/qmail/queue

else
   make setup check  || {
     echo "Kann qmail nicht kompilieren"
     exit 1;
   }  
fi


#
#  2006-06-28 stvo
#  if missing, 421 cannot read controls
#
touch /var/qmail/control/smtpplugins

echo '** config-fast **'

if $HASQMAIL; then
   configfast=/usr/local/pd-admin2/qmail/bin/config-fast
else
   configfast=./config-fast
fi

grep $SERVERNAME /var/qmail/control/locals > /dev/null 2>1 || $configfast $SERVERNAME
cp  /var/qmail/boot/home /var/qmail/rc

if [ -e /usr/lib/sendmail ]
then 
    mv /usr/lib/sendmail /usr/lib/sendmail-orig
fi

if [ -e /usr/sbin/sendmail ]
then
    mv /usr/sbin/sendmail /usr/lib/sendmail-orig
fi

ln -s /var/qmail/bin/sendmail /usr/lib/sendmail
ln -s /var/qmail/bin/sendmail /usr/sbin/sendmail

chown qmaild.qmail /var/qmail/control
if ! $HASQMAIL; then # Ticket 1152350
   chown qmailq.qmail /var/qmail/bin/qmail-queue
   chmod u+s /var/qmail/bin/qmail-queue
fi
chmod 750 /var/qmail/control
chgrp qmail \
/var/qmail/bin/qmail-start \
/var/qmail/bin/qmail-getpw \
/var/qmail/bin/qmail-local \
/var/qmail/bin/splogger \
/var/qmail/bin/qmail-lspawn \
/var/qmail/bin/qmail-rspawn \
/var/qmail/bin/qmail-clean \
/var/qmail/bin/qmail-newu \
/var/qmail/bin/qmail-newmrh \
/var/qmail/bin/qmail-pw2u \
/var/qmail/bin/qmail-inject \
/var/qmail/bin/predate \
/var/qmail/bin/datemail \
/var/qmail/bin/mailsubj \
/var/qmail/bin/qmail-showctl \
/var/qmail/bin/qmail-qread \
/var/qmail/bin/qmail-qstat \
/var/qmail/bin/qmail-tcpto \
/var/qmail/bin/qmail-tcpok \
/var/qmail/bin/qmail-pop3d \
/var/qmail/bin/qmail-send \
/var/qmail/bin/qmail-qmqpc \
/var/qmail/bin/qmail-qmqpd \
/var/qmail/bin/qmail-qmtpd \
/var/qmail/bin/qmail-remote \
/var/qmail/bin/sendmail \
/var/qmail/bin/tcp-env \
/var/qmail/bin/qreceipt \
/var/qmail/bin/qsmhook \
/var/qmail/bin/qbiff \
/var/qmail/bin/forward \
/var/qmail/bin/preline \
/var/qmail/bin/condredirect \
/var/qmail/bin/bouncesaying \
/var/qmail/bin/except \
/var/qmail/bin/maildirmake \
/var/qmail/bin/maildir2mbox \
/var/qmail/bin/maildirwatch \
/var/qmail/bin/qail \
/var/qmail/bin/elq \
/var/qmail/bin/pinq \
/var/qmail/bin/qmail-popup \
/var/qmail/bin/qmail-smtpd

   chmod 755 /var/qmail/bin/qmail-inject
   chmod g+s /var/qmail/bin/qmail-inject
   chmod 755 /var/qmail/bin/sendmail
   chmod g+s /var/qmail/bin/sendmail

###################
### fastforward ###
###################

cd /usr/src
tar -xzf $PFS/fastforward-0.51.tar.gz
cd fastforward-0.51
patch -p1 <<EOT
--- fastforward-0.51/error.h    1998-05-19 18:25:42.000000000 +0200
+++ fastforward-0.51_/error.h   2004-09-08 11:41:43.000000000 +0200
@@ -1,7 +1,9 @@
 #ifndef ERROR_H
 #define ERROR_H

-extern int errno;
+#include <errno.h>
+
+/* extern int errno; */

 extern int error_intr;
 extern int error_nomem;
EOT
make && make setup check || {
    echo "kann fastforward nicht kompilieren"
    exit 1
}

####################
###  spamd starten
####################

mkdir -p /service/spamd || {
	echo "fehler"
	exit 2
}

cat > /service/spamd/run <<EOT
#!/bin/bash
PATH=/usr/local/pd-admin2/bin:$PATH
exec spamd -u spamd
EOT
chmod 755 /service/spamd/run


#####################
####  clamd starten
#####################

mkdir -p /service/clamd || {
        echo "fehler"
        exit 2
}

cat > /service/clamd/run <<EOT
#!/bin/bash
PATH=/usr/local/pd-admin2/bin:/usr/local/pd-admin2/sbin:$PATH
rm -f /tmp/clamd
exec clamd
EOT
chmod 755 /service/clamd/run

####################
###  qmail-scanner
####################

#sc# cd /usr/src
#sc# tar -xzf $PFS/qmail-scanner-1.23.tgz 
#sc# cd qmail-scanner-1.23
#sc# ./configure --scanners fast_spamassassin,clamdscan --debug 0 --batch --notify none --install || {
#sc#    echo "kann qmail-scanner nicht installieren"
#sc#    exit
#sc# }

##########################
####  ezmlm installieren
##########################

cd /usr/src
tar -xzf $PFS/ezmlm-0.53.tar.gz
tar -xzf $PFS/ezmlm-idx-5.1.1.tar.gz
mv ezmlm-idx-5.1.1/* ezmlm-0.53/
cd ezmlm-0.53
patch < idx.patch
echo de > conf-lang
{ echo "cc -O2 -g -fcommon"; cat conf-cc; } >| conf-cc.new
mv conf-cc.new conf-cc

make && make man && make setup || {
    echo "kann ezmlm nicht installieren"
    exit 1
}

###################
### qmail starten
###################

if [ -e /service ]; then
   for i in /service/*; do svc -d $i; done 1>/dev/null 2>&1 
fi


cd /
tar -xzf $PFS/service3r.tar.gz
tar -xzf $PFS/service-dovecot22.tgz
chown -hR root:root /service
sleep 2
mkdir -p /usr/local/lib/pdadmin/mysql/
ln -snf /usr/local/pd-admin2/var/mysql.run/mysql.sock \
   /usr/local/lib/pdadmin/mysql/

/usr/local/pd-admin2/bin/mysqladmin -uroot --password="" shutdown
sleep 4
killall mysqld
sleep 3

if [ -e /service ]; then
   for i in /service/*; do svc -u $i; done 1>/dev/null 2>&1 
fi


mkdir -p /var/qmail/alias
chown alias:qmail /var/qmail/alias
chmod g+w /var/qmail/alias
mkdir -p /var/qmail/users
chown root:qmail /var/qmail/alias


cat <<__ > /var/qmail/control/greylist
[connection]

[auth]

[helo]

[mail]

[rcpt]
/opt/pdadmin/bin/smtp_greylist.pl

[data]
__
